"""Test for the implementation of calc_fields."""

from .physical_constants import r_e
from . import grating
from . import grating_diffuse
import numpy as np
import matplotlib.pyplot as plt
from scipy.special import erf
from genx.gui_logging import iprint

def test_calc_fields_single_interface():
    qz=np.linspace(0.0001, 0.5, 500)
    t, r, k_z=grating.calc_fields(qz/2., 1.54, np.array([9e-5, 9e-5, 0], dtype=np.complex128),
                                  np.array([0.0, 0.0, 0.0]))

    t_theory=2*k_z[:, -1]/(k_z[:, 0]+k_z[:, -1])
    r_theory=(k_z[:, -1]-k_z[:, 0])/(k_z[:, 0]+k_z[:, -1])

    iprint((t_theory-t[:, 0]).max())
    iprint((r_theory-r[:, 1]).max())

    if False:
        plt.subplot(211)
        plt.plot(qz, np.abs(t[:, 0])**2)
        plt.plot(qz, np.abs(t_theory)**2)
        plt.subplot(212)
        plt.semilogy(qz, np.abs(r[:, -1])**2)
        plt.semilogy(np.abs(r_theory)**2)
        plt.show()

def test_calc_fields_single_layer():
    qz=np.linspace(0.0001, 0.5, 500)
    v=np.array([9e-5-2e-6, 5e-5-8e-6, 0], dtype=np.complex128)
    d=np.array([0.0, 100.0, 0.0])
    t, r, k_z=grating.calc_fields(qz/2., 1.54, v, d)
    r1=(k_z[:, 1]-k_z[:, 0])/(k_z[:, 1]+k_z[:, 0])
    r2=(k_z[:, 2]-k_z[:, 1])/(k_z[:, 2]+k_z[:, 1])
    t1=2*k_z[:, 1]/(k_z[:, 0]+k_z[:, 1])
    t2=2*k_z[:, 2]/(k_z[:, 1]+k_z[:, 2])

    r_theory=((r1*np.exp(1.0J*d[1]*k_z[:, 1])/(t1*t2)+r2*np.exp(-1.0J*d[1]*k_z[:, 1])/(t1*t2))/
              (r1*r2*np.exp(1.0J*d[1]*k_z[:, 1])/(t1*t2)+np.exp(-1.0J*d[1]*k_z[:, 1])/(t1*t2)))

    t_lay_theory=t2*np.exp(1.0J*d[1]*k_z[:, 1])/(r1*r2*np.exp(2*1.0J*d[1]*k_z[:, 1])+1.)
    r_lay_theory=r1*t2*np.exp(1.0J*d[1]*k_z[:, 1])/(r1*r2*np.exp(2*1.0J*d[1]*k_z[:, 1])+1.)

    iprint((t_lay_theory-t[:, -2]).max())
    iprint((r_theory-r[:, -1]).max())
    iprint((r_lay_theory-r[:, -2]).max())

    if False:
        plt.subplot(211)
        plt.plot(qz, np.abs(t[:, -2]), 'b:')
        plt.subplot(212)
        plt.plot(qz, np.abs(r[:, -1]), 'g:')
        plt.plot(qz, np.abs(r[:, -2]), 'm:')

        plt.subplot(211)
        plt.plot(qz, np.abs(t_lay_theory), 'b-.')
        plt.subplot(212)
        plt.semilogy(qz, np.abs(r_theory), 'g-.')
        plt.plot(qz, np.abs(r_lay_theory), 'm-.')
        plt.show()

def test_calc_fields_slicing_single_interface():
    lamda=1.54
    v_lay=(2*np.pi/1.54)**2*(1-(1-(1.57-0.4J)*r_e*lamda**2/2/np.pi)**2)
    v_sub=(2*np.pi/1.54)**2*(1-(1-(0.71-0.09J)*r_e*lamda**2/2/np.pi)**2)
    v_layers=np.array([v_lay, v_lay, 0.0])
    sigma=np.array([0., 0., 0.])
    z_int=np.array([100., 0., -20.0])
    # NOTE: The first element has to be the substrate => largest z-value!
    z_slice=np.linspace(150, -40, 160)
    h=0.7

    plt.subplot(211)
    kz_spec=np.arange(0.001, 0.2, 0.001)
    T_1, R_1, kz_1=grating.calc_fields(kz_spec, lamda, v_layers, np.r_[0, z_int[:-1]-z_int[1:]])
    plt.plot(2*kz_spec, np.abs(T_1[:, -2])**2, 'b:')

    v_mean, int_layer_indices=grating_diffuse.calc_layer_prop(z_slice*0+1, sigma, v_layers, z_int, z_slice)
    d=np.r_[0, z_slice[:-1]-z_slice[1:]]
    T_1, R_1, kz_1=grating.calc_fields(kz_spec, lamda, v_mean, d)
    z_bottom_index=np.argmin(np.abs(z_slice-z_int[0]))
    plt.plot(2*kz_spec, np.abs(T_1[:, z_bottom_index])**2, 'g-.')

    plt.subplot(212)
    plt.plot(z_slice, v_mean.real, 'b', z_slice, v_mean.imag, 'g')
    plt.plot(z_slice, np.ones_like(z_slice)*v_lay.real, 'k', z_slice, np.ones_like(z_slice)*v_lay.imag, 'k')
    plt.show()

def test_calc_fields_slicing_single_layer():
    lamda=1.54
    v_lay=(2*np.pi/1.54)**2*(1-(1-(1.57-0.4J)*r_e*lamda**2/2/np.pi)**2)
    v_sub=(2*np.pi/1.54)**2*(1-(1-(0.71-0.09J)*r_e*lamda**2/2/np.pi)**2)
    v_layers=np.array([v_sub, v_lay, 0.0])
    sigma=np.array([0., 0., 0.])
    z_int=np.array([100., 0., -20.0])
    z_slice=np.linspace(150, -40, 190*4+1)

    kz_spec=np.arange(0.001, 0.20, 0.001)
    T_1_lay, R_1_lay, kz_1=grating.calc_fields(kz_spec, lamda, v_layers, np.r_[0, z_int[:-1]-z_int[1:]])

    v_mean, int_layer_indices=grating_diffuse.calc_layer_prop(z_slice*0+1, sigma, v_layers, z_int, z_slice)
    d=np.r_[0, z_slice[:-1]-z_slice[1:]]
    T_1_slice, R_1_slice, kz_1=grating.calc_fields(kz_spec, lamda, v_mean, d)
    z_bottom_index=np.argmin(np.abs(z_slice-z_int[0]))

    # Difficult to get the numbers to agree with high precision due to the slicing.
    iprint((((np.abs(T_1_lay[:, -2])-np.abs(T_1_slice[:, z_bottom_index]))/np.abs(T_1_lay[:, -2])).max()))
    iprint((((np.abs(R_1_lay[:, -2])-np.abs(R_1_slice[:, z_bottom_index]))/np.abs(R_1_lay[:, -2])).max()))
    iprint((((np.abs(R_1_lay[:, -1])-np.abs(R_1_slice[:, -1]))/np.abs(R_1_lay[:, -1])).max()))

    if False:
        plt.subplot(311)
        plt.plot(2*kz_spec, np.abs(T_1_lay[:, -2]), 'b:')
        plt.subplot(312)
        plt.plot(2*kz_spec, np.abs(R_1_lay[:, -1]), 'g:')
        plt.plot(2*kz_spec, np.abs(R_1_lay[:, -2]), 'm:')

        plt.subplot(311)
        plt.plot(2*kz_spec, np.abs(T_1_slice[:, z_bottom_index]), 'b-.')
        plt.subplot(312)
        plt.semilogy(2*kz_spec, np.abs(R_1_slice[:, -1]), 'g-.')
        plt.plot(2*kz_spec, np.abs(R_1_slice[:, z_bottom_index]), 'm-.')

        plt.subplot(313)
        plt.plot(z_slice, v_mean.real, 'b', z_slice, v_mean.imag, 'g')
        plt.plot(z_slice, np.ones_like(z_slice)*v_lay.real, 'k', z_slice, np.ones_like(z_slice)*v_lay.imag, 'k')
        plt.plot(z_slice, np.ones_like(z_slice)*v_sub.real, 'k', z_slice, np.ones_like(z_slice)*v_sub.imag, 'k')
        plt.show()

def test_correlation_function_h05():
    """Test so that the correlation function for h=0.5 matches the analytical solution."""
    eta_r=1050.
    q_r=np.logspace(-5., 0., 100)
    c_func=grating_diffuse.correlation_function(q_r, 0.5, eta_r, 0.0, 1e-10)
    a=1/eta_r
    analytical_func=2.*a/(a**2+q_r**2)

    iprint((np.abs((c_func-analytical_func)/c_func).max()))

    if True:
        plt.loglog(q_r, c_func)
        plt.loglog(q_r, analytical_func)
        plt.show()

def test_marginal_distributions_bvn_int():
    """Test so that the marginal distributions of polynomial_coefficentis_bvn_int is as expected"""
    delta_z_slice_int1=np.linspace(-2.9, 2.9, 200.0)
    delta_z_slice_int2=2.9*np.ones_like(delta_z_slice_int1)
    sigma1=2.0
    sigma2=1.0
    poly=grating_diffuse.polynomial_cofficents_bvn_int(delta_z_slice_int1, delta_z_slice_int2, sigma1, sigma2)
    values=np.polyval(poly, 0.5*np.ones_like(delta_z_slice_int1))

    ref_values=0.5*(1+erf(delta_z_slice_int1/np.sqrt(2)/sigma1))

    iprint((np.abs((values-ref_values)/ref_values).max()))

    if True:
        plt.plot(delta_z_slice_int1, values)
        plt.plot(delta_z_slice_int1, ref_values)
        plt.show()

# test_calc_fields()
# test_calc_fields_single_layer()
# test_calc_fields_slicing_single_interface()
# test_calc_fields_slicing_single_layer()
test_correlation_function_h05()
# test_marginal_distributions_bvn_int()



''' A library for handling optical constants and scattering lengths
in an efficent way. Based on a base class database which is subclassed
for each case.
Programmer: Matts Bjorck
Last changed: 2009-03-19
'''

import numpy as np
import re, string
# Imports needed for the dispersive table implementation
from scipy import interpolate
from . import refl
from genx.gui_logging import iprint

# ==============================================================================
class Func(object):
    '''A function object which stores the real function so it can be 
    dynamically replaced from its parents.
    '''

    def __init__(self, func):
        self.func=func

    def __call__(self, *args):
        return self.func(*args)

    def replace_func(self, func):
        self.func=func

class Proxy(object):
    '''Proxy class borrowed from ASPN Recipe 496741 at adress
    http://code.activestate.com/recipes/496741/
    '''
    __slots__=["_obj", "__weakref__"]

    def __init__(self, obj):
        object.__setattr__(self, "_obj", obj)

    #
    # proxying (special cases)
    #
    def __getattribute__(self, name):
        return getattr(object.__getattribute__(self, "_obj"), name)

    def __delattr__(self, name):
        delattr(object.__getattribute__(self, "_obj"), name)

    def __setattr__(self, name, value):
        setattr(object.__getattribute__(self, "_obj"), name, value)

    def __bool__(self):
        return bool(object.__getattribute__(self, "_obj"))

    def __str__(self):
        return str(object.__getattribute__(self, "_obj"))

    def __repr__(self):
        return repr(object.__getattribute__(self, "_obj"))

    def __call__(self, *args, **kwargs):
        return object.__getattribute__(self, "_obj").__call__(*args, **kwargs)

    def _change_object(self, obj):
        object.__setattr__(self, "_obj", obj)

    #
    # factories
    #
    _special_names=[
        '__abs__', '__add__', '__and__', '__cmp__', '__coerce__',
        '__contains__', '__delitem__', '__delslice__', '__div__', '__divmod__',
        '__eq__', '__float__', '__floordiv__', '__ge__', '__getitem__',
        '__getslice__', '__gt__', '__hash__', '__hex__', '__iadd__', '__iand__',
        '__idiv__', '__idivmod__', '__ifloordiv__', '__ilshift__', '__imod__',
        '__imul__', '__int__', '__invert__', '__ior__', '__ipow__', '__irshift__',
        '__isub__', '__iter__', '__itruediv__', '__ixor__', '__le__', '__len__',
        '__long__', '__lshift__', '__lt__', '__mod__', '__mul__', '__ne__',
        '__neg__', '__oct__', '__or__', '__pos__', '__pow__', '__radd__',
        '__rand__', '__rdiv__', '__rdivmod__', '__reduce__', '__reduce_ex__',
        '__repr__', '__reversed__', '__rfloorfiv__', '__rlshift__', '__rmod__',
        '__rmul__', '__ror__', '__rpow__', '__rrshift__', '__rshift__', '__rsub__',
        '__rtruediv__', '__rxor__', '__setitem__', '__setslice__', '__sub__',
        '__truediv__', '__xor__', 'next',
        ]

    # @classmethod #This is the new way of doing a classmethod, starting from python 2.4
    def _create_class_proxy(cls, theclass):
        """creates a proxy for the given class"""

        def make_method(name):
            def method(self, *args, **kw):
                return getattr(object.__getattribute__(self, "_obj"), name)(*args, **kw)

            return method

        namespace={}
        for name in cls._special_names:
            if hasattr(theclass, name) and not hasattr(cls, name):
                namespace[name]=make_method(name)
        return type("%s(%s)"%(cls.__name__, theclass.__name__), (cls,), namespace)

    # And, this is the old way. Which I stck with for compatiblity reasons
    _create_class_proxy=classmethod(_create_class_proxy)

    def __new__(cls, obj, *args, **kwargs):
        """
        creates an proxy instance referencing `obj`. (obj, *args, **kwargs) are
        passed to this class' __init__, so deriving classes can define an 
        __init__ method of their own.
        note: _class_proxy_cache is unique per deriving class (each deriving
        class must hold its own cache)
        """
        try:
            cache=cls.__dict__["_class_proxy_cache"]
        except KeyError:
            cls._class_proxy_cache=cache={}
        try:
            theclass=cache[obj.__class__]
        except KeyError:
            cache[obj.__class__]=theclass=cls._create_class_proxy(obj.__class__)
        ins=object.__new__(theclass)
        # theclass.__init__(ins, obj, *args, **kwargs)
        return ins

def change_proxyobject(proxy, obj):
    '''
    changes the current object of proxy to the obj.
    Note that evil things can happen if not care is taken so the objects are
    of the same class
    '''
    object.__setattr__(proxy, '_obj', obj)

# ==============================================================================
class Database(object):
    '''A database class where object memebers are dynamically acessed
    and stored. I.e. the constants are only looked up when needed.
    '''

    def __init__(self):
        ''' __init__(self) --> None
        '''
        object.__setattr__(self, 'stored_values', {})

    def __getattribute__(self, name):
        '''__getattr__(self, name) --> object
        
        Looks up and returns the attribure name
        '''
        if name.startswith('__'):
            return object.__getattribute__(self, name)
        name=name.lower()
        stored_values=object.__getattribute__(self, 'stored_values')
        if name in stored_values:
            return stored_values[name]
        else:
            try:
                stored_values[name]=object.__getattribute__(self,
                                                            'lookup_value')(name)
            except (LookupError, IOError) as e:
                raise LookupError('The name %s does not exist in the' \
                                  'database'%name)
            return stored_values[name]

    def __setattr__(self, name, value):
        '''__setattr__(self, name) --> None
        
        Just overlaoding the setattribute so a object cant be set.
        '''
        raise TypeError('This object does not support assignements')

    def lookup_value(self, name):
        '''lookup_value(self, name) --> object
        
        Used to (externally) lookup a value in a database to be inserted in
        local one for this object.
        '''
        iprint('Looking up value')
        return 1

    def reset_database(self):
        '''reset_database(self) --> None
        
        Resets the internal database
        '''
        stored_values=object.__getattribute__(self, 'stored_values')
        stored_values={}

# ==============================================================================
class FormFactor(Database):
    ''' A database for the x-ray formfactor which includes the 
    anomulous part as well as the angle dependent part. The object will
    return a function of sin(theta)/lambda
    '''

    def __init__(self, wavelength, f_calc):
        '''__init__(self, wavelength, f_calc) --> None
        
        wavelength [float] the wavelength of the radiation in AA, f_calc a 
        function which calculates (lookup the scattering factor)
        '''
        Database.__init__(self)
        object.__setattr__(self, 'wavelength', wavelength)
        object.__setattr__(self, 'f_calc', f_calc)

    def set_wavelength(self, wavelength):
        '''set_wavelength(self, wavelength) --> None
        
        sets the wavelength of the database. This will reset the 
        all the values. I.e when called each value will be reloaded
        '''
        # First check so we actually change the wavelength ...
        if abs(wavelength \
               -object.__getattribute__(self, 'wavelength'))>1e-10:
            object.__setattr__(self, 'wavelength', wavelength)
            # object.__getattribute__(self, 'reset_database')()
            stored_vals=object.__getattribute__(self, 'stored_values')
            # for key in stored_vals:
            #    f = object.__getattribute__(self, 'lookup_value')(key)
            #    change_proxyobject(stored_vals[key], f)
            # Removing a bug with proxy adding does not work properly
            for key in stored_vals:
                f=object.__getattribute__(self, 'lookup_value')(key)
                stored_vals[key]=f

    def __getattribute__(self, name):
        '''__getattribute__(self, name) --> value
        
        Finds and returns a attribute
        '''
        if name=='set_wavelength' or name.startswith('__'):
            return object.__getattribute__(self, 'set_wavelength')
        else:
            # print 'getting value'
            return Database.__getattribute__(self, name)

    def lookup_value(self, name):
        '''lookup_value(self, name) --> f [function]
        
        looks up a value in the external database
        '''
        wl=object.__getattribute__(self, 'wavelength')
        # f = Proxy((object.__getattribute__(self, 'f_calc')(name, wl)))
        # Removing a bug with proxy adding does not work properly
        f=object.__getattribute__(self, 'f_calc')(name, wl)
        return f

# ==============================================================================
class ScatteringLength(Database):
    ''' A simpler version of the database where everything is loaded at once
    good for simple and small databases. I.e the scatteringlength for
    a neutron which is largely wavelength independent.
    '''

    def __init__(self, values):
        '''__init__(self, values) --> None
        
        values is a dictonary of key value pairs that are used in the database.
        '''
        Database.__init__(self)
        object.__setattr__(self, 'stored_values', values)

    def lookup_value(self, name):
        raise LookupError('The element %s does not exist in the database'%name)

# ==============================================================================
# Function to load databases and or values


def load_f0dabax(filename, create_rho=False):
    '''load_dabax(filename) --> f0 (dictonary)
    
    loads a dabax file with f0 values and return a dictonary with 
    f0(sin(theta)/lambda) for the element given by the key name.
    '''

    def create_f(a, b, c):
        def f(sin_over_lambda):
            '''f(sin_over_lambda) --> float
            
            The atomic form factor for x-rays. The non-dispertive part.
            '''
            return c+sum(a[:, np.newaxis]*np.exp(-b[:, np.newaxis] \
                                                 *sin_over_lambda**2), 0)

        return f

    def create_rho(a, b, c):
        def rho(r, fp, wb, occ):
            return (sum(a[:, np.newaxis]/
                        (8.0*(np.pi*(b[:, np.newaxis]/16/np.pi**2+wb))**1.5)
                        *np.exp(-(r**2.0)/4.0/(b[:, np.newaxis]/16/np.pi**2+wb)),
                        0)+(c+fp)/(8.0*(np.pi*wb)**1.5)*
                    np.exp(-(r**2.0)/4.0/wb))*occ

        return rho

    with open(filename, 'r') as f:
        real_label=''
        temp_dict={}
        for line in f.readlines():
            # Get the label for each line
            if line[0]=='#':
                label=line[1]
                ret=line[1:-1]
            else:
                label='D'
                ret=line[:-1]
            # Gets the real label, atom name
            if label=='S':
                real_label=ret.split()[-1]
            # The row contains data
            if label=='D':
                temp_dict[real_label.lower()]=[float(x) for x in ret.split()]

    f0={}
    rho0={}
    for key in list(temp_dict.keys()):
        temp=temp_dict[key]
        # change the name to not have operators...
        if key[-1]=='-':
            key=key[:-1]+'m'
        if key[-1]=='+':
            key=key[:-1]+'p'
        f0[key]=create_f(np.array(temp[:4]), np.array(temp[5:]), temp[4])
        rho0[key]=create_rho(np.array(temp[:4]), np.array(temp[5:]), temp[4])
    if create_rho:
        return f0, rho0
    else:
        return f0

def create_fp_lookup(path):
    '''create_f_lookup(filename) --> lookup_func(name, wavelength)
    
    Creates a lookup function to lookup element names and returns a function
    that yields dispersive scattering factors at Q = 0. NOTE wavelengths in AA
    '''

    def lookup_func(name, wavelength):
        '''lookup_func(name, wavelength) --> fp = f1 - 1.0J*f2
        
        looksup the scattering factors for a given wavelength given in AA for
        a element (note that the databases does not support ions). 
        The data given is the dispersive part + f0 (non dispersive) at Q = 0.
        '''
        e, f1, f2=np.loadtxt(path+'%s.nff'%name.lower(), skiprows=1,
                             unpack=True)
        energy=1239.842/wavelength*10
        if energy>=e[-2] or energy<=e[1]:
            raise ValueError('The energy/wavelength is outside the databse' \
                             +'range, the energy should be inside [%f,%f] '%(e[1], e[-2]))
        pos1=np.argmin(abs(e-energy))
        # Is the energy point to the right or left of the current point
        # If it is ontop it doesn't matter since the interpolation will be exact at
        # the endpoints
        if (e[pos1]-energy)>0:
            pos2=pos1-1
        else:
            pos2=pos1+1
        # A quick linear interpolation:
        f1_e=(energy-e[pos1])*(f1[pos2]-f1[pos1])/(e[pos2]-e[pos1]) \
             +f1[pos1]
        f2_e=(energy-e[pos1])*(f2[pos2]-f2[pos1])/(e[pos2]-e[pos1]) \
             +f2[pos1]

        return f1_e-1.0J*f2_e

    return lookup_func

def create_fpdisp_lookup(path):
    '''create_f_lookup(filename) --> lookup_func(name)

    Creates a lookup function to lookup element names and returns a function
    that yields a function of dispersive scattering factors f(E) at Q = 0. NOTE energy is in eV
    '''

    def create_dispersion_func(name):
        e, f1, f2=np.loadtxt(path+'%s.nff'%name.lower(), skiprows=1,
                             unpack=True)
        f1interp=interpolate.interp1d(e, f1, kind='linear')
        f2interp=interpolate.interp1d(e, f2, kind='linear')

        def f(energy):
            return f1interp(energy)-1.0J*f2interp(energy)

        return refl.ReflFunction(f, (np.mean(e),), {}, id='f(E)')

    return create_dispersion_func

def create_f_lookup(lookup_fp, f0):
    '''create_f_lookup(lookup_fp, f0) --> lookup_func(name, wavelength)
    
    combines a f0 dictonary [dict] witht the lookup function for the
     anoumoulous scattering factors. Note that it is assumed that lookup_fp 
    is a function on the form lookup_fp(name, wavelength) --> f1 - 1.0Jf2
    where name is the name of the element!
    '''

    def lookup_func(name, wavelength):
        ''' Looks up the total angle dependent form factor 
        f = f0 + f1 + 1.0J*f2 of element name. This dispersive part
        is independent on the ionicity but f0 not.
        '''
        # Check if name corrspond to an ion
        element=None
        if len(name)>1:
            if name[-2].isdigit():
                element=name[:-2]
            else:
                element=name
        else:
            element=name
        # Remove the non dispersive part for the given element
        fp=lookup_fp(element, wavelength)-f0[element](0)
        f0_name=f0[name]

        def f(sin_over_lambda):
            return f0_name(sin_over_lambda)+fp

        return f

    return lookup_func

def create_rho_lookup(lookup_fp, rho0, f0):
    '''create_rho_lookup(lookup_fp, rho0, f0) --> lookup_func(name, wavelength)
    
    combines a rho0 dictonary [dict] with the lookup function for the
     anoumoulous scattering factors. Note that it is assumed that lookup_fp 
    is a function on the form lookup_fp(name, wavelength) --> f1 - 1.0Jf2
    where name is the name of the element!
    '''

    def lookup_func(name, wavelength):
        ''' Looks up the "electron density" of an atom as described by
        its formfactor, f = f0 + f1 + 1.0J*f2, of element name.
        The dispersive part is independent on the ionicity but f0 not.
        '''
        # Check if name corrspond to an ion
        element=None
        if len(name)>1:
            if name[-2].isdigit():
                element=name[:-2]
            else:
                element=name
        else:
            element=name
        # Remove the non dispersive part for the given element
        fp=lookup_fp(element, wavelength)-f0[element](0)
        rho0_name=rho0[name]

        def rho(r, wb, occ):
            return rho0_name(r, fp, wb, occ)

        return rho

    return lookup_func

def create_fw_lookup(lookup_fp, weight):
    '''create_fw_lookup(lookup_fp, weight) --> lookup_func(name, wavelength)
    
    combines a f0 dictonary [dict] witht the lookup function for the
     anoumoulous scattering factors. Note that it is assumed that lookup_fp 
    is a function on the form lookup_fp(name, wavelength) --> f1 - 1.0Jf2
    where name is the name of the element!
    '''

    def lookup_func(name, wavelength):
        ''' Looks up the total form factor at Q = 0
        f = f1 + 1.0J*f2 of element name. 
        '''
        fp=lookup_fp(name, wavelength)
        w=weight[name]

        return fp/(w/0.6022141)

    return lookup_func

def load_bdabax(filename):
    '''load_bdabax(filename) --> b (dictonary)
    
    loads a dabax file with b (sld for neutrons) values 
    and return a dictonary with b for the elements and isotopes 
    given by the key name.
    '''
    temp_dict=read_dabax(filename)
    # We are intrested in the scatttering lengths
    b_c={}
    for key in temp_dict:
        val=temp_dict[key][1]+temp_dict[key][2]*1.0J
        if key[0].isdigit():
            b_c['i'+key]=val
        else:
            b_c[key]=val

    return b_c

def load_fdabax(filename):
    ''' loads a dabax file with the scattering length tables returns a lookup 
    function so that the wavelength can be changed. '''
    table=read_dabax(filename)

    def lookup_func(name, wavelength):
        ''' Looks up the total form factor at Q = 0
        f = f1 + 1.0J*f2 of element name. 
        '''
        el=np.array(table[name])
        el=el.reshape(len(el)/7, 7)
        e=el[:, 0]
        f1=el[:, 1]
        f2=el[:, 2]
        energy=1239.842/wavelength*10/1e3  # keV
        if energy>=e[-2] or energy<=e[1]:
            raise ValueError('The energy/wavelength is outside the databse' \
                             +'range, the energy should be inside [%f,%f] '%(e[1], e[-2]))
        pos1=np.argmin(abs(e-energy))
        # Is the energy point to the right or left of the current point
        # If it is ontop it doesn't matter since the interpolation will be exact at
        # the endpoints
        if (e[pos1]-energy)>0:
            pos2=pos1-1
        else:
            pos2=pos1+1
        # A quick linear interpolation:
        f1_e=(energy-e[pos1])*(f1[pos2]-f1[pos1])/(e[pos2]-e[pos1]) \
             +f1[pos1]
        f2_e=(energy-e[pos1])*(f2[pos2]-f2[pos1])/(e[pos2]-e[pos1]) \
             +f2[pos1]

        return f1_e-1.0J*f2_e

    return lookup_func

def read_dabax(filename):
    '''read_dabax_dict(filename) --> temp_dict [dict]
    
    Loads an entire dabax file to a dictonary. Scan names are the 
    keys.
    '''

    def tofloat(x):
        try:
            f=float(x)
        except:
            f=np.nan
        return f

    real_label=''
    temp_dict={}
    with open(filename) as f:
        for line in f.readlines():
            # Get the label for each line
            if line[0]=='#':
                label=line[1]
                ret=line[1:-1]
            else:
                label='Data'
                ret=line[:-1]
            # Gets the real label, atom name
            if label=='S':
                real_label=ret.split()[-1]
            # The row contains data
            if label=='Data':
                # To get all values in the table
                if real_label.lower() not in temp_dict:
                    temp_dict[real_label.lower()]=[tofloat(x.split('(')[0]) for x in ret.split()]
                else:
                    temp_dict[real_label.lower()]+=[tofloat(x.split('(')[0]) for x in ret.split()]

    return temp_dict

def load_atomic_weights_dabax(filename):
    '''load__atomic_weights_dabax(filename) --> w (dictonary)
    
    loads a dabax file with w (sld for neutrons) values 
    and return one dictonary with the atomic weights for the elements 
    by the key name. Note that all isotopes are includes with the an
    i in front of them.
    '''

    temp_dict=read_dabax(filename)
    # We are intrested in the scatttering lengths
    w_mean={}
    for key in temp_dict:
        if key[0].isdigit():
            # This is an isotope
            w_mean['i'+key]=temp_dict[key][0]
            if len(temp_dict[key])==3:
                # The mean atomic mass for the element
                w_mean[key.lstrip(string.digits)]=temp_dict[key][2]

    return w_mean

def create_scatt_weight(scatt_dict, w_dict):
    '''create_bw(scatt_dict, w_dict) --> sw_dict
    
    Makes a scattering length database for using with densities (g/cm3)
    '''
    sw_dict={}
    for key in scatt_dict:
        # print key, ' ',scatt_dict[key], ' ',w_dict[key]
        if key in w_dict:
            sw_dict[key]=scatt_dict[key]/complex(w_dict[key]/0.6022141)
    return sw_dict



"""
Classes to interface with the bumps module for fitting and statistical analysis.
"""

import wx
from wx.grid import Grid, EVT_GRID_CELL_LEFT_DCLICK
import threading
from numpy import newaxis, maximum
from matplotlib.colors import LogNorm
from bumps import __version__ as bumps_version
from bumps.monitor import TimedUpdate
from bumps.fitproblem import nllf_scale
from bumps.formatnum import format_uncertainty
from bumps.dream.corrplot import _hists, _plot
from .plotpanel import PlotPanel

class ProgressMonitor(TimedUpdate):
    """
    Display fit progress on the console
    """

    def __init__(self, problem, pbar, ptxt, progress=0.25, improvement=5.0):
        TimedUpdate.__init__(self, progress=progress, improvement=improvement)
        self.problem=problem
        self.pbar=pbar  # wx.Gauge
        self.ptxt=ptxt  # wx.StaticText
        self.chis=[]
        self.steps=[]

    def show_progress(self, history):
        scale, err=nllf_scale(self.problem)
        chisq=format_uncertainty(scale*history.value[0], err)
        self.ptxt.SetLabel('step: %s/%s\tcost: %s'%(history.step[0], self.pbar.GetRange(), chisq))
        self.pbar.SetValue(history.step[0])
        self.steps.append(history.step[0])
        self.chis.append(scale*history.value[0])

    def show_improvement(self, history):
        return
        p=self.problem.getp()
        try:
            self.problem.setp(history.point[0])
            out='<table width="50%"><tr>'
            out+="</tr>\n<tr>".join(
                ["<td>%s</td><td>%s</td><td>%s</td>"%(pi.name, pi.value, pi.bounds) for pi in self.problem._parameters])
            self.result_text.value=out+"</td></tr></table>"
        finally:
            self.problem.setp(p)
        with self.plot_out:
            fig=figure()
            plot(self.steps, self.chis)
            clear_output()
            display(fig)
            close()

class NBMonitor(TimedUpdate):
    """
    Display fit progress on the console
    """

    def __init__(self, problem, progress=0.25, improvement=5.0):
        TimedUpdate.__init__(self, progress=progress, improvement=improvement)
        from ipywidgets import Output, HTML, HBox, VBox
        from IPython.display import display
        self.problem=problem
        self.status_text=HTML()
        self.result_text=HTML()
        self.plot_out=Output()

        vbox=VBox([self.status_text, self.result_text])
        hbox=HBox([vbox, self.plot_out])
        display(hbox)
        self.steps=[]
        self.chis=[]

    def show_progress(self, history):
        scale, err=nllf_scale(self.problem)
        chisq=format_uncertainty(scale*history.value[0], err)
        self.status_text.value='<table width="50%%"><tr><td>step</td><td>%s</td><td>cost</td><td>%s</td></tr></table>'%(
        history.step[0], chisq)
        self.steps.append(history.step[0])
        self.chis.append(scale*history.value[0])

    def show_improvement(self, history):
        from IPython.display import display, clear_output
        from matplotlib.pyplot import figure, plot, close
        p=self.problem.getp()
        try:
            self.problem.setp(history.point[0])
            out='<table width="50%"><tr>'
            out+="</tr>\n<tr>".join(
                ["<td>%s</td><td>%s</td><td>%s</td>"%(pi.name, pi.value, pi.bounds) for pi in self.problem._parameters])
            self.result_text.value=out+"</td></tr></table>"
        finally:
            self.problem.setp(p)
        with self.plot_out:
            fig=figure()
            plot(self.steps, self.chis)
            clear_output()
            display(fig)
            close()

class StatisticalAnalysisDialog(wx.Dialog):
    rel_cov=None

    def __init__(self, parent, model):
        wx.Dialog.__init__(self, parent, style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER | wx.MAXIMIZE_BOX)
        vbox=wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(vbox)
        self.SetTitle('Statistical Analysis of Parameters')

        dpi_scale_factor=wx.GetApp().dpi_scale_factor

        self.ptxt=wx.StaticText(self, label="...")
        self.pbar=wx.Gauge(self, range=1000)

        hbox=wx.BoxSizer(wx.HORIZONTAL)
        vbox.Add(hbox, proportion=1, flag=wx.EXPAND)

        lpanel=wx.Panel(self)
        gpanel=wx.Panel(self)
        gbox=wx.BoxSizer(wx.VERTICAL)
        gpanel.SetSizer(gbox)
        self.grid=Grid(gpanel)
        gbox.Add(wx.StaticText(gpanel, label='Estimated covariance matrix:'),
                 proportion=0, flag=wx.FIXED_MINSIZE)
        gbox.Add(self.grid, proportion=1, flag=wx.EXPAND)
        self.normalize_checkbox=wx.CheckBox(gpanel, label='Normalize value (σ_ij/σ_i/σ_j)')
        self.normalize_checkbox.SetValue(True)
        self.normalize_checkbox.Bind(wx.EVT_CHECKBOX, self.OnToggleNormalize)
        gbox.Add(self.normalize_checkbox, proportion=0, flag=wx.FIXED_MINSIZE)

        nfparams=len([pi for pi in model.parameters if pi.fit])
        self.grid.CreateGrid(nfparams+1, nfparams)
        self.grid.SetColLabelTextOrientation(wx.VERTICAL)
        self.grid.SetColLabelSize(int(dpi_scale_factor*80))
        for i in range(nfparams):
            self.grid.SetRowSize(i, int(dpi_scale_factor*80))
            self.grid.SetColSize(i, int(dpi_scale_factor*80))
            self.grid.SetColLabelValue(i, '%i'%i)
            self.grid.SetRowLabelValue(i+1, '%i'%i)
        self.grid.SetRowSize(nfparams, int(dpi_scale_factor*80))
        self.grid.DisableCellEditControl()
        self.grid.SetMinSize((int(dpi_scale_factor*200),int(dpi_scale_factor*200)))
        self.grid.Bind(EVT_GRID_CELL_LEFT_DCLICK, self.OnSelectCell)
        rpanel=PlotPanel(self)
        rpanel.SetMinSize((int(dpi_scale_factor*200),int(dpi_scale_factor*200)))
        self.plot_panel=rpanel
        self.ax=rpanel.figure.add_subplot(111)

        hbox.Add(lpanel, proportion=0, flag=wx.EXPAND|wx.ALIGN_TOP)
        hbox.Add(gpanel, proportion=1, flag=wx.EXPAND)
        hbox.Add(rpanel, proportion=1, flag=wx.EXPAND)


        lsizer=wx.GridSizer(vgap=1, hgap=2, cols=2)
        lpanel.SetSizer(lsizer)
        self.entries={}
        for key, emin, emax, val in [('pop', 1, 20*nfparams, 2*nfparams),
                                     ('samples', 1000, 10000000, 10000),
                                     ('burn', 0, 10000, 200)]:
            lsizer.Add(wx.StaticText(lpanel, label='%s:'%key), flag=wx.FIXED_MINSIZE)
            self.entries[key]=wx.SpinCtrl(lpanel, wx.ID_ANY, min=emin, max=emax, value=str(val))
            lsizer.Add(self.entries[key], flag=wx.FIXED_MINSIZE)

        lsizer.AddStretchSpacer(10)
        but=wx.Button(self, label='Run Analysis...')
        vbox.Add(but)
        vbox.Add(self.ptxt, proportion=0, flag=wx.EXPAND)
        vbox.Add(self.pbar, proportion=0, flag=wx.EXPAND)

        self.Bind(wx.EVT_BUTTON, self.OnRunAnalysis, but)
        self.Bind(wx.EVT_CLOSE, self.OnClose)

        self.model=model
        self.thread=None

        psize=parent.GetSize()
        self.SetSize(int(psize.GetWidth()*0.75), int(psize.GetHeight()*0.75))

    def OnRunAnalysis(self, event):
        if self.thread is not None:
            return
        self.thread=threading.Thread(target=self.run_bumps)
        self.thread.start()

    def run_bumps(self):
        self.bproblem=self.model.bumps_problem()
        mon=ProgressMonitor(self.bproblem, self.pbar, self.ptxt)
        pop=self.entries['pop'].GetValue()
        burn=self.entries['burn'].GetValue()
        samples=self.entries['samples'].GetValue()
        self.pbar.SetRange(int(samples/(len(self.bproblem.model_parameters())*pop))+burn)

        res=self.model.bumps_fit(method='dream',
                                 pop=pop, samples=samples, burn=burn,
                                 thin=1, alpha=0, outliers='none', trim=False,
                                 monitors=[mon], problem=self.bproblem)
        self._res=res
        wx.CallAfter(self.display_bumps)

    def display_bumps(self):
        self.thread.join(timeout=5.0)
        self.thread=None
        self.pbar.SetValue(0)

        res=self._res
        self.draw=res.state.draw()
        self.abs_cov=res.cov
        self.rel_cov=res.cov/res.dx[:,newaxis]/res.dx[newaxis,:]
        rel_max=[0, 1, abs(self.rel_cov[0,1])]
        if self.normalize_checkbox.IsChecked():
            display_cov=self.rel_cov
            fmt="%.6f"
        else:
            display_cov=self.abs_cov
            fmt="%.4g"
        self.grid.SetRowLabelValue(0, 'Value/Error:')
        for i, ci in enumerate(self.rel_cov):
            self.grid.SetColLabelValue(i, self.draw.labels[i])
            self.grid.SetRowLabelValue(i+1, self.draw.labels[i])
            self.grid.SetCellValue(0, i, "%.8g\n%.4g"%(res.x[i], res.dx[i]))
            self.grid.SetCellAlignment(0, i, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE)
            self.grid.SetReadOnly(0, i)
            self.grid.SetCellBackgroundColour(0, i, "#cccccc")
            for j, cj in enumerate(ci):
                self.grid.SetCellValue(i+1, j, fmt%display_cov[i,j])
                self.grid.SetReadOnly(i+1, j)
                self.grid.SetCellAlignment(i+1, j, wx.ALIGN_CENTRE, wx.ALIGN_CENTRE)
                if i==j:
                    self.grid.SetCellBackgroundColour(i+1, j, "#888888")
                elif abs(cj)>0.4:
                    self.grid.SetCellBackgroundColour(i+1, j, "#ffcccc")
                elif abs(cj)>0.3:
                    self.grid.SetCellBackgroundColour(i+1, j, "#ffdddd")
                elif abs(cj)>0.2:
                    self.grid.SetCellBackgroundColour(i+1, j, "#ffeeee")
                if i!=j and abs(cj)>rel_max[2]:
                    rel_max=[min(i, j), max(i, j), abs(cj)]
        self.hists=_hists(self.draw.points.T, bins=50)

        fig=self.plot_panel.figure
        fig.clear()
        ax=fig.add_subplot(111)
        data, x, y=self.hists[(rel_max[0], rel_max[1])]
        vmin, vmax=data[data>0].min(), data.max()
        ax.pcolorfast(y, x, maximum(vmin, data), norm=LogNorm(vmin, vmax), cmap='inferno')
        ax.set_xlabel(self.draw.labels[rel_max[1]])
        ax.set_ylabel(self.draw.labels[rel_max[0]])
        self.plot_panel.flush_plot()

        # add analysis data do model for later storange in export header
        exdict={}
        exdict['library']='bumps'
        exdict['version']=bumps_version
        exdict['settings']=dict(pop=self.entries['pop'].GetValue(),
                                burn=self.entries['burn'].GetValue(),
                                samples=self.entries['samples'].GetValue())
        exdict['parameters']=[
            dict(name=li, value=float(xi), error=float(dxi),
                 cross_correlations=dict((self.draw.labels[j], float(res.cov[i,j]))
                                         for j in range(len(res.x))))
            for i, (li, xi, dxi) in enumerate(zip(self.draw.labels, res.x, res.dx))
            ]
        self.model.extra_analysis['statistics_mcmc']=exdict

    def OnToggleNormalize(self, evt):
        if self.rel_cov is None:
            evt.Skip()
            return
        if self.normalize_checkbox.IsChecked():
            display_cov=self.rel_cov
            fmt="%.6f"
        else:
            display_cov=self.abs_cov
            fmt="%.4g"
        for i, ci in enumerate(self.rel_cov):
            for j, cj in enumerate(ci):
                self.grid.SetCellValue(i+1, j, fmt%display_cov[i,j])

    def OnSelectCell(self, evt):
        i, j=evt.GetCol(), evt.GetRow()-1
        if i==j or j==-1:
            return
        elif i>j:
            itmp=i
            i=j
            j=itmp

        fig=self.plot_panel.figure
        fig.clear()
        ax=fig.add_subplot(111)
        data, x, y=self.hists[(i,j)]
        vmin, vmax=data[data>0].min(), data.max()
        ax.pcolorfast(y, x, maximum(vmin, data), norm=LogNorm(vmin, vmax), cmap='inferno')
        ax.set_xlabel(self.draw.labels[j])
        ax.set_ylabel(self.draw.labels[i])
        self.plot_panel.flush_plot()

    def OnClose(self, event):
        if self.thread is not None and self.thread.is_alive():
            # a running bumps simulation can't be interrupted from other thread, stop window from closing
            self.ptxt.SetLabel("Simulation running")
            event.Veto()
            return
        event.Skip()


"""
Define a class to hold all data for ORSO data files.
"""

import os
import json
import warnings
import numpy as np
import datetime
from collections import OrderedDict
from . import ORSOStandardError, ORSOStandardWarning

try:
    # read header schema for validation
    import jsonschema

    SCHEMA=json.load(
        open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'reduced_data_header_schema.json'), 'r'))
    validator=jsonschema.Draft7Validator(SCHEMA)
except Exception as err:
    warnings.warn("Can't validate headers: %s"%str(err), UserWarning)
    validator=None

def validate_header(header, strict=True):
    if validator is None:
        return True
    valid=validator.is_valid(header)
    if valid:
        return True
    text='Header does not comply with ORSO standard:\n'
    for err in validator.iter_errors(header):
        text+='  path /%s: %s\n'%('/'.join(err.absolute_path), err.message)
    if strict:
        raise ORSOStandardError(text)
    else:
        warnings.warn(text, ORSOStandardWarning)
        return False

def get_schema_from_path(node_path):
    node=SCHEMA
    for pi in node_path:
        node=node[pi]
    return node

def get_schema_child_nodes(node_path):
    node=get_schema_from_path(node_path)
    items=[(key, value, node_path+[key]) for key, value in node['properties'].items() if key in node['required']]
    output={}
    for key, value, npath in items:
        # make sure the item is not a reference or has multiple options
        while not 'type' in value:
            if 'anyOf' in value:
                # if there are several choices, use the first
                value=value['anyOf'][0]
                npath.append(0)
            elif '$ref' in value:
                npath=value['$ref'][2:].split('/')
                value=get_schema_from_path(npath)
            else:
                raise ValueError("Could not find correct item to referenc, path="+'/'.join(node_path))
        if value['type']=='object':
            output[key]=get_schema_child_nodes(npath)
        elif value['type']=='string':
            if 'enum' in value:
                output[key]=value['enum'][0]
            elif 'format' in value and value['format']=='date-time':
                output[key]=datetime.datetime.now().strftime('%Y-%m-%dT%H:%M:%S')
            else:
                output[key]='unspecified'
        elif value['type']=='number':
            output[key]=0.
    return output

class DataColumn(np.ndarray):
    name=None
    unit=None
    dimension=None

    def __new__(cls, data, name='unset', unit=None, dimension=None, min_size=None, **opts):
        adata=np.array(data, dtype=np.float64, **opts)
        if adata.shape[0]<min_size:
            appends=np.zeros(min_size-adata.shape[0], dtype=np.float64)
            adata=np.hstack([adata, appends])
        out=adata.view(cls)
        out.name=name
        out.unit=unit
        out.dimension=dimension
        return out

    def __repr__(self):
        output=np.ndarray.__repr__(self)[:-1]
        if self.name is not None:
            output+=', name="%s"'%self.name
        if self.unit is not None:
            output+=', unit="%s"'%self.unit
        if self.dimension is not None:
            output+=', dimension="%s"'%self.dimension
        return output+')'

class ORSOData():
    """
    Representation of a reflectometry dataset with metadata.
    According to ORSO standard definition.
    """

    def __init__(self, header, data, strict=True):
        if len(data)<3:
            raise ORSOStandardError("Need at least 3 data columns, Qz, R, dR")
        # make sure the data consists of DataColumn arrays with same length
        maxlen=max([len(di) for di in data])
        self._data=[DataColumn(di, min_size=maxlen) for di in data]
        if 'columns' in header and len(header['columns'])==len(data):
            for i, ci in enumerate(header['columns']):
                try:
                    self._data[i].name=ci['name']
                except KeyError:
                    raise ORSOStandardError("Each column needs a name")
                if 'unit' in ci:
                    self._data[i].unit=ci['unit']
                if 'dimension' in ci:
                    self._data[i].dimension=ci['dimension']
        else:
            raise ORSOStandardError("Header requires 'columns' field entry for every data column")
        validate_header(header, strict)
        self.header=OrderedDict(header)
        self.header.move_to_end('columns')

    @classmethod
    def minimal_header(cls):
        # return a minimal OrderedDict that fulfills the SCHEMA
        output=OrderedDict(get_schema_child_nodes([]))
        output['columns']=[{'name': 'Qz'}, {'name': 'R'}, {'name': 'sR'}]
        return output

    @property
    def x(self):
        return self._data[0]

    @property
    def y(self):
        return self._data[1]

    @property
    def dy(self):
        return self._data[2]

    @property
    def dx(self):
        if len(self._data)>3 and self._data[3].name=='sQz':
            return self._data[3]
        else:
            return None

    @property
    def columns(self):
        return [di.name for di in self._data]

    @property
    def data(self):
        return np.vstack(self._data)

    @property
    def units(self):
        return [di.unit if di.unit is not None else '1' for di in self._data]

    def __getitem__(self, item):
        return self._data[item]

    def __repr__(self):
        output='ORSOData('
        output+='columns=%s,\n'%repr(self.columns)
        output+='         units=%s'%repr(self.units)
        return output+') '+self.name

    def __eq__(self, other):
        output=(self.header==other.header)
        for i, col in enumerate(self):
            output&=(col.shape==other[i].shape)
            if not output:
                return False
            output&=bool((col==other[i]).all())
        return output

    @property
    def name(self):
        if 'data_set' in self.header:
            return self.header['data_set']
        else:
            return ''

    def plot(self, axes=None):
        if axes is None:
            from matplotlib import pylab
            axes=pylab.gca()
        if self.dx is None:
            axes.errorbar(self.x, self.y, yerr=self.dy, label=self.name)
        else:
            axes.errorbar(self.x, self.y, yerr=self.dy, xerr=self.dx, label=self.name)
        axes.yscale('log')
        axes.xlabel('%s (%s)'%(self.x.name, self.x.unit))
        axes.ylabel('%s (%s)'%(self.x.name, self.x.unit))



__author__='GenX'

import string

import wx

# inspection does not work as vtk mangles with the python miport system
# noinspection PyUnresolvedReferences
from vtk import vtkRenderer, vtkActor, vtkTextMapper, vtkActor2D, vtkSphereSource, vtkPolyDataMapper, vtkAxesActor
# noinspection PyUnresolvedReferences
import vtk.util.colors as vtkc

from genx.gui_logging import iprint
from .wxVTKRenderWindow import wxVTKRenderWindow
from . import atom_colors as atom_colors
from . import custom_dialog
from . import sxrd_images
from functools import reduce

class VTKview(wxVTKRenderWindow):
    def __init__(self, parent):
        wxVTKRenderWindow.__init__(self, parent, wx.NewId(), stereo=0)
        self.ren=vtkRenderer()
        self.GetRenderWindow().AddRenderer(self.ren)
        self.ren.SetBackground(0, 0, 0)
        self.sphereActor=vtkActor()

        # Settings for creation of sample
        self.x_uc=1
        self.y_uc=1
        self.use_sym=True
        self.fold_sym=True

        # Some defualts
        self.radius=1.0
        self.theta_res=15
        self.phi_res=15
        self.amb_col=(1., 1., 1.)
        self.diffuse=1.0
        self.ambient=0.2
        self.specular=0.5
        self.specular_power=50.
        self.specular_col=(1., 1., 1.)
        self.element_col=atom_colors.jmol
        self.default_col=vtkc.yellow

        self.search_radius=0.1

        self.toolbar=None
        self.cursor_mode='orbit'

        self.textMapper=vtkTextMapper()
        tprop=self.textMapper.GetTextProperty()
        tprop.SetFontFamilyToArial()
        tprop.SetFontSize(10)
        tprop.BoldOn()
        tprop.ShadowOn()
        tprop.SetColor(1, 1, 1)
        self.textActor=vtkActor2D()
        self.textActor.VisibilityOff()
        self.textActor.SetMapper(self.textMapper)

        self.ren.AddActor(self.textActor)

    def do_toolbar(self, parent):
        """Create and return a toolbar that can be used to control the widget"""
        toolbar=wx.ToolBar(parent, style=wx.TB_FLAT | wx.TB_VERTICAL)
        dpi_scale_factor=wx.GetApp().dpi_scale_factor
        tb_bmp_size=int(dpi_scale_factor*20)

        button_names=['View X', 'View Y', 'View Z', 'Isometric']
        button_images=[sxrd_images.x, sxrd_images.y, sxrd_images.z, sxrd_images.isometric]
        callbacks=[self.OnViewX, self.OnViewY, self.OnViewZ, self.OnViewIsometric]
        tooltips=['View along X', 'View along Y', 'View along Z', 'Isometric view']

        for i in range(len(button_names)):
            new_id=wx.NewId()
            toolbar.AddTool(new_id, label=button_names[i],
                            bitmap=wx.Bitmap(button_images[i].GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                            shortHelp=tooltips[i])
            parent.Bind(wx.EVT_TOOL, callbacks[i], id=new_id)

        button_names=['Select', 'Orbit', 'Zoom', 'Pan']
        button_images=[sxrd_images.selection, sxrd_images.orbit, sxrd_images.zoom_small, sxrd_images.pan]
        callbacks=[self.OnChangeCursorState, self.OnChangeCursorState, self.OnChangeCursorState,
                   self.OnChangeCursorState]
        tooltips=['Object selection', 'Orbit', 'Zoom', 'Pan']

        self.cursor_ids=[]
        for i in range(len(button_names)):
            new_id=wx.NewId()
            self.cursor_ids.append(new_id)
            # print button_images[i].GetBitmap()
            toolbar.AddCheckTool(new_id, button_names[i],
                                 wx.Bitmap(button_images[i].GetImage().Scale(tb_bmp_size, tb_bmp_size)),
                                 shortHelp=tooltips[i])
            parent.Bind(wx.EVT_TOOL, callbacks[i], id=new_id)

        toolbar.ToggleTool(self.cursor_ids[1], True)
        self.toolbar=toolbar

        return toolbar

    def OnViewX(self, event):
        """Aligns the view with the x axis"""
        if self._CurrentRenderer:
            self._CurrentCamera.SetFocalPoint(0, 0, 0)
            self._CurrentCamera.SetPosition(1.0, 0.0, 0.0)
            self._CurrentCamera.SetViewUp(0.0, 0.0, 1.0)

            # self._CurrentCamera.OrthogonalizeViewUp()
            self._CurrentRenderer.ResetCamera()
            self.Render()

    def OnViewY(self, event):
        """Aligns the view with the y axis"""
        if self._CurrentRenderer:
            self._CurrentCamera.SetFocalPoint(0, 0, 0)
            self._CurrentCamera.SetPosition(0.0, 1.0, 0.0)
            self._CurrentCamera.SetViewUp(0.0, 0.0, 1.0)
            self._CurrentRenderer.ResetCamera()
            self.Render()

    def OnViewZ(self, event):
        """Alings the view with the z axis"""
        if self._CurrentRenderer:
            self._CurrentCamera.SetFocalPoint(0, 0, 0)
            self._CurrentCamera.SetPosition(0.0, 0.0, 1.0)
            self._CurrentCamera.SetViewUp(0.0, 1.0, 0.0)
            self._CurrentRenderer.ResetCamera()
            self.Render()

    def OnViewIsometric(self, event):
        """Creates an isometric view"""
        if self._CurrentRenderer:
            self._CurrentCamera.SetFocalPoint(0, 0, 0)
            self._CurrentCamera.SetPosition(1.0, 1.0, 1.0)
            self._CurrentCamera.SetViewUp(0.0, 0.0, 1.0)
            self._CurrentRenderer.ResetCamera()
            self.Render()

    def OnChangeCursorState(self, event):
        """Callback when changing the cursor state between select, orbit, zoom and pan"""
        iprint(self.toolbar)
        if self.toolbar:
            [self.toolbar.ToggleTool(cid, False) for cid in self.cursor_ids]
            self.toolbar.ToggleTool(event.GetId(), True)
            name=self.toolbar.FindById(event.GetId()).GetLabel()
            if self.cursor_mode=='select':
                self.textActor.VisibilityOff()
                self.sphereActor.VisibilityOff()
                self.Render()
            if name=='Select':
                self.cursor_mode='select'
            elif name=='Orbit':
                self.cursor_mode='orbit'
            elif name=='Zoom':
                self.cursor_mode='zoom'
            elif name=='Pan':
                self.cursor_mode='pan'
            else:
                iprint('VTKView.OnChangeCursorState: Button name ', name, 'is not a known button')
            # print name, self.cursor_mode

    def highlight(self, actor):
        # outline = vtk.vtkOutlineFilter()
        # print dir(actor)
        # print dir(actor.GetProperty())

        sphere=vtkSphereSource()
        sphere.SetRadius(self.radius*1.1)
        sphere.SetCenter(actor.GetCenter())
        sphere.SetThetaResolution(self.theta_res)
        sphere.SetPhiResolution(self.phi_res)

        spheremapper=vtkPolyDataMapper()
        spheremapper.SetInputConnection(sphere.GetOutputPort())

        sphereActor=vtkActor()
        sphereActor.SetMapper(spheremapper)
        sphereActor.GetProperty().SetColor((0.0, 0.0, 0.0))
        sphereActor.GetProperty().SetOpacity(0.5)

        # sphereActor.GetProperty().SetRepresentationToWireframe()
        try:
            self.ren.RemoveViewProp(self.sphereActor)
        except Exception as e:
            iprint(e)
        self.ren.AddActor(sphereActor)
        self.sphereActor=sphereActor
        self.Render()

    def OnLeftDown(self, event):
        """Overriding the defualt on left down"""
        if self.cursor_mode=='zoom':
            self._Mode="Zoom"
        elif self.cursor_mode=='pan':
            self._Mode="Pan"
        elif self.cursor_mode=='orbit':
            self._Mode="Rotate"
        elif self.cursor_mode=='select':
            self.OnSelectAtom(event)
            event.Skip()

    def OnSelectAtom(self, event):
        """ Callback for  the selection of an atom"""
        if self._CurrentRenderer:
            x=event.GetX()
            y=event.GetY()

            renderer=self._CurrentRenderer
            picker=self._Picker

            windowX, windowY=self._RenderWindow.GetSize()

            picker.Pick(x, (windowY-y-1), 0.0, renderer)
            actor=picker.GetActor()
            if not picker.GetActor():
                self.textActor.VisibilityOff()
                self.sphereActor.VisibilityOff()
                self.Render()
                # print 'Could not locate an atom'
            else:
                selPt=picker.GetSelectionPoint()
                pickPos=picker.GetPickPosition()

                x, y, z=actor.GetCenter()
                self.textMapper.SetInput("%s"%reduce(lambda x, y: x+','+y, self.locate_ids(x, y, z)))
                self.textActor.SetPosition(picker.GetSelectionPoint()[:2])
                self.textActor.VisibilityOn()
                self.highlight(actor)

    def OnRightDown(self, event):
        ''' Overriding defualt Pick Actor '''
        if not self._Mode:
            if event.ControlDown():
                self._Mode="Rotate"
            elif event.ShiftDown():
                self._Mode="Pan"
            else:
                self._Mode="Zoom"

    def locate_ids(self, x, y, z):
        ''' Given the picked positions x,y,z locate the strings ids that
            are within a distance of self.search_radius.
            '''
        within_dist=(((self.atom_x-x)**2+(self.atom_y-y)**2+(self.atom_z-z)**2)
                     <self.search_radius**2)
        ids=[]
        for i, pick in enumerate(within_dist):
            if pick and not self.atom_ids[i] in ids:
                ids.append(self.atom_ids[i])
        return ids

    def _get_col(self, element):
        '''Translate element to color according to element dict.
        '''
        # We need to take care if the eleemnt is an ion.
        # the two last chars are on the form Xp or Xm where X is a digit
        element=element.lower()
        if len(element)>2:
            if element[-2] in string.digits:
                element=element[:-2]
        if element in self.element_col:
            # print 'Found: ', element
            col=self.element_col[element]
        else:
            # print 'Missing: ', element
            col=self.default_col
        return col

    def clear_view(self):
        '''remove all actors so a new sample can be built.
        '''
        self.ren.RemoveAllViewProps()
        self.ren.AddActor2D(self.textActor)
        self.textActor.VisibilityOff()

    def create_axes(self, a=2.0, b=2.0, c=2.0):
        axesActor=vtkAxesActor()
        # axesActor.SetShaftTypeToCylinder()
        # axesActor.SetXAxisLabelText('x')
        # axesActor.SetYAxisLabelText('y')
        # axesActor.SetZAxisLabelText('z')
        axesActor.SetTotalLength(a, b, c)
        self.ren.AddActor(axesActor)

    def _create_sphere(self, x, y, z, col, opacity=1.0):
        '''Create a sphere actor at (x,y,z) with color c and return it'''
        sphere=vtkSphereSource()
        sphere.SetRadius(self.radius)
        sphere.SetCenter(x, y, z)
        sphere.SetThetaResolution(self.theta_res)
        sphere.SetPhiResolution(self.phi_res)

        spheremapper=vtkPolyDataMapper()
        spheremapper.SetInputConnection(sphere.GetOutputPort())

        sphereActor=vtkActor()
        sphereActor.SetMapper(spheremapper)
        sphereActor.GetProperty().SetDiffuseColor(col)
        sphereActor.GetProperty().SetAmbientColor(self.amb_col)
        sphereActor.GetProperty().SetDiffuse(self.diffuse)
        sphereActor.GetProperty().SetAmbient(self.ambient)
        sphereActor.GetProperty().SetSpecular(self.specular)
        sphereActor.GetProperty().SetSpecularPower(self.specular_power)
        sphereActor.GetProperty().SetSpecularColor(self.specular_col)
        sphereActor.GetProperty().SetOpacity(opacity)
        return sphereActor

    def add_atom(self, x, y, z, id, element, opacity=1.0):
        col=self._get_col(element)
        sphere_actor=self._create_sphere(x, y, z, col, opacity)
        self.ren.AddActor(sphere_actor)

    def build_sample(self, sample, use_opacity=False):
        '''Builds an sample view from the object sample that has to
        have the method _surf_pars that yields the x, y, z, u, oc, el
        '''
        self.clear_view()
        x, y, z, u, oc, el, ids=sample.create_uc_output(x_uc=self.x_uc, y_uc=self.y_uc, use_sym=self.use_sym,
                                                        fold_sym=self.fold_sym)
        self.atom_x=x
        self.atom_y=y
        self.atom_z=z
        self.atom_ids=ids
        if use_opacity:
            [self.add_atom(x[i], y[i], z[i], 'None', el[i], oc[i]) for i in range(len(x))]
        else:
            [self.add_atom(x[i], y[i], z[i], 'None', el[i], 1.0) for i in range(len(x))]
        self.create_axes()

        self.ren.ResetCamera()
        self.Render()

    def Show(self, *args):
        self.Render()

    def show(self):
        self.Reset()
        self.Render()

    def ShowSettingDialog(self):
        """Shows a settings dialog to change the settings"""
        parameters=['Use symmetry', 'Fold symmetry op', 'a unit cell rep.', 'b unit cell rep.', 'Atom radius']
        values={'Use symmetry': self.use_sym, 'Fold symmetry op': self.fold_sym, 'a unit cell rep.': self.x_uc,
                'b unit cell rep.': self.y_uc, 'Atom radius': self.radius}
        units={'Use symmetry': '', 'Fold symmetry op': '', 'a unit cell rep.': 'uc',
               'b unit cell rep.': 'uc', 'Atom radius': 'AA'}
        validators={'Use symmetry': True, 'Fold symmetry op': True, 'a unit cell rep.': 1,
                    'b unit cell rep.': 1, 'Atom radius': custom_dialog.FloatObjectValidator()}
        groups=[['Unit cell', ('Use symmetry', 'Fold symmetry op', 'a unit cell rep.',
                               'b unit cell rep.')], ['Rendering', ('Atom radius',)]]

        dlg=custom_dialog.ValidateDialog(self, parameters, values, validators, units=units, groups=groups,
                                         title="Domain Viewer Settings")
        if dlg.ShowModal()==wx.ID_OK:
            new_values=dlg.GetValues()
            self.use_sym=bool(new_values['Use symmetry'])
            self.fold_sym=bool(new_values['Fold symmetry op'])
            self.x_uc=int(new_values['a unit cell rep.'])
            self.y_uc=int(new_values['b unit cell rep.'])
            self.radius=float(new_values['Atom radius'])



"""
Constants used in various computations.
"""

r_e=2.8179403227e-5 # classical electron radius [Å] ( 1/(4 pi epsilon0) * (e_charge**2)/(m_e*c**2) )

M_to_SLD=2.853e-6 # m/A/Å² conversion M [A/m] to SLD [Å^-2]
ApM_to_emucc=1e-3 # conversion M [A/m] to M [emu/cm³]
Memu_to_SLD=M_to_SLD*ApM_to_emucc
muB=9.2740101E-24 # Bohr magneton [A·m^2]

muB_to_SL=muB*M_to_SLD*1e24 # magnetic scattering length for a moment of 1 Bohr magneton
AAm2_to_emucc=1e-5/Memu_to_SLD # conversion of SLD [Å/m²] to M [emu/cm³]




#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""
.. module:: parameters
   :synopsis: Classes to work with differnt materials in scattering experiments

.. moduleauthor:: Matts Björck <matts.bjorck@gmail.com>

"""

from os import path
import re
import pickle

import numpy as np
from scipy import interpolate

from .parameters import HasParameters, ComplexArray, Float, Calc
from .physical_constants import r_e
from . import scatteringlengths as _sl

# Transformations between AA and eV
_AA_to_eV=12398.42

_module_dir, _filename=path.split(__file__)
# Look only after the file name and not the ending since
# the file ending can be pyc if compiled...
_filename=_filename.split('.')[0]
_module_dir='.' if _module_dir=='' else _module_dir

_atomic_weights=_sl.load_atomic_weights_dabax(path.join(_module_dir, "../databases/AtomicWeights.dat"))

def create_fpdisp_lookup(path):
    """ Creates a lookup function to lookup element names and returns a function
    that yields a function of dispersive scattering factors f(E) at Q = 0. NOTE energy is in eV
    """

    def create_dispersion_func(name):
        e, f1, f2=np.loadtxt(path+'%s.nff'%name.lower(), skiprows=1, unpack=True)
        f1interp=interpolate.interp1d(e, f1, kind='linear')
        f2interp=interpolate.interp1d(e, f2, kind='linear')
        def_wl=_AA_to_eV/np.mean(e)

        @MemoizeF
        def f(wl=def_wl, **kwargs):
            return f1interp(_AA_to_eV/wl)-1.0J*f2interp(_AA_to_eV/wl)

        return Calc(f)

    return create_dispersion_func

f0=create_fpdisp_lookup(path.join(_module_dir, "../databases/f1f2_nist/"))

class MemoizeF:
    """Remember previous evaluations of function fn and stores it in a dictionary.

    Based on the Cookbook recipe: Memoizing (Caching) the Return Values of Functions in
    Python Cookbook by David Ascher, Alex Martelli.
    """

    def __init__(self, fn):
        self.fn=fn
        self.memo={}

    def __call__(self, wl=1.54, **kwds):
        pickled=pickle.dumps(wl, 1)
        if pickled not in self.memo:
            self.memo[pickled]=self.fn(wl=wl, **kwds)
        return self.memo[pickled]

class Material(HasParameters):
    """ Class to keep transform a chemical formula and mass density to scattering length

    The formula is given in ASCII format, for example 'Ag2(NO2)3' or 'HNO3', nested parentheses are allowed.
    Also non-integer indices are allowed for example Fe0.01Pd

    """
    validation_kwargs={'wl': 1.54}

    # The scattering length should be in AA-2
    # Need to transform g/cm3 to formula units per AA3
    # 1u = 1.66054×10−27 kg 1 AA=10−10m
    # density[kg/m3] = 1.66054*10^3*density[u/A3]
    # density [g/cm3] = density [kg/m3] * 1e-3/1e-2**3 = density [kg/m3] * 1e3
    # So formula units per AA^ = density [kg/m3]/1.66054*10^3/weight per formula unit
    # or formula units per AA^ = density [g/cm3]/1.66054/weight per formula unit

    def __init__(self, formula, **kwargs):
        self.sld_x=ComplexArray(0, unit="1/AA-2", help="Scattering length density")
        self.density=Float(0.0, unit="g/cm3", help="Weight density")
        self._formula_density=Float(0.0, unit="u", help="weight of one formula unit")
        HasParameters.__init__(self, **kwargs)
        self.formula_dict={}
        self.parse_formula(formula)

        # Create the composition variables needed to describe formula
        w_list=[]
        f0_list=[]
        for name in self.formula_dict:
            self.__setattr__(name, Float(self.formula_dict[name], help="Number of %s atoms"%name))
            element_count=self.__getattribute__(name)
            w_list.append(element_count*self.get_atomic_weight(name))
            f0_list.append(element_count*self.get_f0(name))
        self._formula_density=self.density/1.66054/sum(w_list)
        self.sld_x=self._formula_density*sum(f0_list)*r_e

    def get_atomic_weight(self, element):
        """Returns the atomic weight in unified atomic mass units (u)

        Parameters:
            element(string): name of the element

        """
        return _atomic_weights[element.lower()]

    def get_f0(self, element):
        """Returns the complex atomic forward scattering length normalised to the scattering length of one electron for
         element.

        Parameters:
            element(string): name of the element

        Returns:
            f0(function): function for f0 that takes the wavelength as a keyword argument
        """
        return f0(element)

    def parse_formula(self, formula, multiplier=1):
        """Parses the string formula and returns a list of isotopes

        Parameters:
            formula(string): A string containing the formula.
            multiplier(int): A number representing that will be used to multiply the element count
        """
        # regexp = re.compile(r"([A-Z][a-z]?)(\d*)|\((.*)\)(\d*)")
        regexp=re.compile(r"([A-Z][a-z]?)(\d*\.?\d*)|\((.*)\)(\d*\.?\d*)")

        element, element_count, group, group_count=(1, 2, 3, 4)
        pos=0
        while pos<len(formula):
            match=regexp.match(formula, pos)
            if not match:
                raise ValueError("Error in formula: %s at position %d"%(formula, pos))
            pos=match.end()
            if match.group(element):
                # print "Match %s is an element" % match.group(0)
                count=self.formula_count(match.group(element_count), multiplier)
                try:
                    self.formula_dict[match.group(element)]+=count
                except KeyError:
                    self.formula_dict[match.group(element)]=count
            elif match.group(group):
                # print "Group found: %s, count: %s" % (match.group(group), match.group(group_count))
                count=self.formula_count(match.group(group_count), multiplier)
                self.parse_formula(match.group(group), multiplier=count)

    def formula_count(self, count_str, multiplier):
        """Handles the parsing result for the count of an formula part. Returns the count*multiplier"

        Parameters:
            count_str(string): The count of the element/group as a string
            multiplier(float): The  multiplier from previous groups.

        """
        if not count_str:
            # Set count to 1 if count is not exist, ie a single group is parsed
            count=1
        else:
            count=float(count_str)
        return count*multiplier

if __name__=="__main__":
    test=Material("Ag1.1(NO0.01)2", density=7.87)
    print(_atomic_weights['fe'])
    # print test.Ag.help
    print(test._formula_density(), 2/2.88**3)
    print(test.sld_x(wl=1.54))
    test.Fe=2
    print(test.sld_x(wl=1.54))
    test.Fe=1
    print(test.sld_x(wl=1.54))
    print(test.sld_x(wl=15.4))
    print(test.sld_x(wl=15.4))



'''File: diffev.py an implementation of the differential evolution algoithm
for fitting.
Programmed by: Matts Bjorck
Last changed: 2008 11 23
'''

from numpy import *
from .gui_logging import iprint
from logging import debug
import _thread
import time
import random as random_mod
import sys, os, pickle

__mpi_loaded__=False
__parallel_loaded__=False
_cpu_count=1

try:
    import multiprocessing as processing

    __parallel_loaded__=True
    _cpu_count=processing.cpu_count()
except ImportError:
    debug('processing not installed no parallel processing possible')
    processing=None

try:
    from mpi4py import MPI as mpi
except ImportError:
    rank=0
    __mpi_loaded__=False
    mpi=None
else:
    __mpi_loaded__=True
    comm=mpi.COMM_WORLD
    size=comm.Get_size()
    rank=comm.Get_rank()

from . import model as mmodel

from .lib.Simplex import Simplex

# Add current path to the system paths
# just in case some user make a directory change
sys.path.append(os.getcwd())

# ==============================================================================
# class: DiffEv
class DiffEv:
    '''
    Class DiffEv
    Contains the implemenetation of the differential evolution algorithm.
    It also contains thread support which is activated by the start_fit 
    function.
    '''

    export_parameters={'km': float, 'kr': float, 'pf': float, 'use_pop_mult': bool, 'pop_mult': int, 'pop_size': int,
                       'use_max_generations': bool, 'max_generations': int, 'max_generation_mult': int,
                       'use_start_guess': bool, 'use_boundaries': bool, 'sleep_time': float,
                       'use_parallel_processing': bool, 'use_mpi': bool, 'fom_log': array, 'start_guess': array,
                       'limit_fit_range': bool, 'fit_xmin': float, 'fit_xmax': float,
                       }

    parameter_groups=[
        ['Fitting', ['use_start_guess', 'use_boundaries', 'use_autosave', 'autosave_interval']],
        ['Diff. Ev.', ['km', 'kr', 'method']],
        ['Population size', ['use_pop_mult', 'pop_mult', 'pop_size']],
        ['Max. Generatrions', ['use_max_generations', 'max_generations', 'max_generation_mult']],
        ['Parallel processing', ['use_parallel_processing', 'processes', 'chunksize']],
        ]

    def create_mutation_table(self):
        # Mutation schemes implemented
        self.mutation_schemes=[self.best_1_bin, self.rand_1_bin,
                               self.best_either_or, self.rand_either_or, self.jade_best, self.simplex_best_1_bin]

    def __init__(self):
        self.create_mutation_table()

        self.model=mmodel.Model()

        self.km=0.7  # Mutation constant
        self.kr=0.7  # Cross over constant
        self.pf=0.5  # probablility for mutation
        self.c=0.07
        self.simplex_interval=5  # Interval of running the simplex opt
        self.simplex_step=0.05  # first step as a fraction of pop size
        self.simplex_n=0.0  # Number of individuals that will be optimized by simplex
        self.simplex_rel_epsilon=1000  # The relative epsilon - convergence critera
        self.simplex_max_iter=100  # THe maximum number of simplex runs
        # Flag to choose beween the two alternatives below
        self.use_pop_mult=False
        self.pop_mult=3  # Set the pop_size to pop_mult * # free parameters
        self.pop_size=10  # Set the pop_size only

        # Flag to choose between the two alternatives below
        self.use_max_generations=False
        self.max_generations=500  # Use a fixed # of iterations
        self.max_generation_mult=6  # A mult const for max number of iter

        # Flag to choose whether or not to use a starting guess
        self.use_start_guess=True
        # Flag to choose wheter or not to use the boundaries
        self.use_boundaries=True

        # Sleeping time for every generation
        self.sleep_time=0.2
        # Allowed disagreement between the two different fom
        # evaluations
        self.fom_allowed_dis=1e-10
        # Flag if we should use parallel processing 
        self.use_parallel_processing=__parallel_loaded__*0
        if __parallel_loaded__:
            self.processes=_cpu_count
        else:
            self.processes=0

        self.chunksize=1

        # Flag for using mpi
        self.use_mpi=False

        # Flag for using autosave
        self.use_autosave=True
        # autosave interval in generations        
        self.autosave_interval=10

        # Functions that are user definable
        self.plot_output=default_plot_output
        self.text_output=default_text_output
        self.parameter_output=default_parameter_output
        self.autosave=defualt_autosave
        self.fitting_ended=default_fitting_ended

        # Definition for the create_trial function
        self.create_trial=self.best_1_bin
        self.update_pop=self.standard_update_pop
        self.init_new_generation=self.standard_init_new_generation

        # Control flags:
        self.running=False  # true if optimization is running
        self.stop=False  # true if the optimization should stop
        self.setup_ok=False  # True if the optimization have been setup
        self.error=False  # True/string if an error ahs occured

        # Logging variables
        # Maximum number of logged elements
        self.max_log=100000
        self.fom_log=array([[0, 0]])[0:0]
        # self.par_evals = array([[]])[0:0]

        self.par_evals=CircBuffer(self.max_log, buffer=array([[]])[0:0])
        # self.fom_evals = array([])
        self.fom_evals=CircBuffer(self.max_log)

        self.start_guess=array([])

        self.limit_fit_range=False
        self.fit_xmin=0.01
        self.fit_xmax=0.1

    @property
    def method(self):
        return self.create_trial.__name__

    @method.setter
    def method(self, value):
        names=self.methods
        if value in names:
            self.create_trial=self.mutation_schemes[names.index(value)]
        else:
            raise ValueError("Mutation method has to be in %s"%names)

    @property
    def methods(self):
        return [f.__name__ for f in self.mutation_schemes]

    def write_h5group(self, group, clear_evals=False):
        """ Write parameters into hdf5 group

        :param group: h5py Group to write into
        :return:
        """
        for par in self.export_parameters:
            obj=getattr(self, par)
            group[par]=obj

        if clear_evals:
            group['par_evals']=self.par_evals.array()[0:0]
            group['fom_evals']=self.fom_evals.array()[0:0]
        else:
            group['par_evals']=self.par_evals.array()
            group['fom_evals']=self.fom_evals.array()

    def read_h5group(self, group):
        """ Read parameters from a hdf5 group

        :param group: h5py Group to read from
        :return:
        """
        self.setup_ok=False
        for par in self.export_parameters:
            obj=getattr(self, par)
            if self.export_parameters[par] is array:
                setattr(self, par, group[par][()])
            else:
                try:
                    setattr(self, par, self.export_parameters[par](group[par][()]))
                except KeyError:
                    iprint("No value found for %s"%par)
                    continue

        self.par_evals.copy_from(group['par_evals'][()])
        self.fom_evals.copy_from(group['fom_evals'][()])

    def safe_copy(self, object):
        '''safe_copy(self, object) --> None
        
        Does a safe copy of object to this object. Makes copies of everything 
        if necessary. The two objects become decoupled.
        '''
        self.km=object.km  # Mutation constant
        self.kr=object.kr  # Cross over constant
        self.pf=object.pf  # probablility for mutation

        # Flag to choose beween the two alternatives below
        self.use_pop_mult=object.use_pop_mult
        self.pop_mult=object.pop_mult
        self.pop_size=object.pop_size

        # Flag to choose between the two alternatives below
        self.use_max_generations=object.use_max_generations
        self.max_generations=object.max_generations
        self.max_generation_mult=object.max_generation_mult

        # Flag to choose whether or not to use a starting guess
        self.use_start_guess=object.use_start_guess
        # Flag to choose wheter or not to use the boundaries
        self.use_boundaries=object.use_boundaries

        # Sleeping time for every generation
        self.sleep_time=object.sleep_time
        # Flag if we should use parallel processing 
        if __parallel_loaded__:
            self.use_parallel_processing=object.use_parallel_processing
        else:
            self.use_parallel_processing=False

        print("copy")
        try:
            self.limit_fit_range=object.limit_fit_range
            self.fit_xmin=object.fit_xmin
            self.fit_xmax=object.fit_xmax
        except AttributeError:
            self.limit_fit_range=False

        # Flag if we should use mpi
        if __mpi_loaded__:
            try:
                self.use_mpi=object.use_mpi
            except AttributeError:
                self.use_mpi=False
        else:
            self.use_mpi=False

        # Definition for the create_trial function
        # self.set_create_trial(object.get_create_trial())

        # True if the optimization have been setup
        self.setup_ok=object.setup_ok

        # Logging variables
        self.fom_log=object.fom_log[:]
        self.par_evals.copy_from(object.par_evals)
        self.fom_evals.copy_from(object.fom_evals)

        if self.setup_ok:
            self.n_pop=object.n_pop
            self.max_gen=object.max_gen

            # Starting values setup
            self.pop_vec=object.pop_vec

            self.start_guess=object.start_guess

            self.trial_vec=object.trial_vec
            self.best_vec=object.best_vec

            self.fom_vec=object.fom_vec
            self.best_fom=object.best_fom
            # Not all implementaions has these copied within their files
            # Just ignore if an error occur
            try:
                self.n_dim=object.n_dim
                self.par_min=object.par_min
                self.par_max=object.par_max
            except:
                pass

    def pickle_string(self, clear_evals=False):
        '''Pickle the object.

        Saves a copy into a pickled string note that the dynamic
        functions will not be saved. For normal use this is taken care of
        outside this class with the config object.
        '''
        cpy=DiffEv()
        cpy.safe_copy(self)
        if clear_evals:
            cpy.par_evals.buffer=cpy.par_evals.buffer[0:0]
            cpy.fom_evals.buffer=cpy.fom_evals.buffer[0:0]
        cpy.create_trial=None
        cpy.update_pop=None
        cpy.init_new_generation=None
        cpy.plot_output=None
        cpy.text_output=None
        cpy.parameter_output=None
        cpy.autosaves=None
        cpy.fitting_ended=None
        cpy.model=None
        cpy.mutation_schemes=None

        return pickle.dumps(cpy)

    def pickle_load(self, pickled_string):
        '''load_pickles(self, pickled_string) --> None
        
        Loads the pickled string into the this object. See pickle_string.
        '''
        obj=pickle.loads(pickled_string, encoding='latin1', errors='ignore')
        obj.create_mutation_table()
        self.safe_copy(obj)

    def reset(self):
        ''' reset(self) --> None
        
        Resets the optimizer. Note this has to be run if the optimizer is to
        be restarted.
        '''
        self.setup_ok=False

    def connect_model(self, model):
        '''connect_model(self, model) --> None
        
        Connects the model [model] to this object. Retrives the function
        that sets the variables  and stores a reference to the model.
        '''
        # Retrive parameters from the model
        (par_funcs, start_guess, par_min, par_max)=model.get_fit_pars()

        # Control parameter setup
        self.par_min=array(par_min)
        self.par_max=array(par_max)
        self.par_funcs=par_funcs
        self.model=model
        self.n_dim=len(par_funcs)
        model.limit_fit_range, model.fit_xmin, model.fit_xmax=(
            self.limit_fit_range,
            self.fit_xmin,
            self.fit_xmax)
        if not self.setup_ok:
            self.start_guess=start_guess

    def init_fitting(self, model):
        '''
        Function to run before a new fit is started with start_fit.
        It initilaize the population and sets the limits on the number
        of generation and the population size.
        '''
        self.connect_model(model)
        if self.use_pop_mult:
            self.n_pop=int(self.pop_mult*self.n_dim)
        else:
            self.n_pop=int(self.pop_size)
        if self.use_max_generations:
            self.max_gen=int(self.max_generations)
        else:
            self.max_gen=int(self.max_generation_mult*self.n_dim*self.n_pop)

        # Starting values setup
        self.pop_vec=[self.par_min+random.rand(self.n_dim)*(self.par_max-self.par_min)
                      for i in range(self.n_pop)]

        if self.use_start_guess:
            self.pop_vec[0]=array(self.start_guess)

        self.trial_vec=[zeros(self.n_dim) for i in range(self.n_pop)]
        self.best_vec=self.pop_vec[0]

        self.fom_vec=zeros(self.n_dim)
        self.best_fom=1e20

        # Storage area for JADE archives
        self.km_vec=ones(self.n_dim)*self.km
        self.kr_vec=ones(self.n_dim)*self.kr

        # Logging varaibles
        self.fom_log=array([[0, 1]])[0:0]
        self.par_evals=CircBuffer(self.max_log, buffer=array([self.par_min])[0:0])
        # self.fom_evals = array([])
        self.fom_evals=CircBuffer(self.max_log)
        # Number of FOM evaluations
        self.n_fom=0
        # self.par_evals.reset(array([self.par_min])[0:0])
        # self.fom_evals.reset()

        if rank==0:
            self.text_output('DE initilized')

        # Remeber that everything has been setup ok
        self.setup_ok=True

    def init_fom_eval(self):
        '''init_fom_eval(self) --> None
        
        Makes the eval_fom function
        '''
        model=self.model

        # Setting up for parallel processing
        if self.use_parallel_processing and __parallel_loaded__:
            self.text_output('Setting up a pool of workers ...')
            self.setup_parallel()
            self.eval_fom=self.calc_trial_fom_parallel
        elif self.use_mpi and __mpi_loaded__:
            self.setup_parallel_mpi()
            self.eval_fom=self.calc_trial_fom_parallel_mpi
        else:
            self.eval_fom=self.calc_trial_fom

    def start_fit(self, model):
        '''
        Starts fitting in a seperate thred.
        '''
        # If it is not already running
        if not self.running:
            # Initilize the parameters to fit
            self.reset()
            self.init_fitting(model)
            self.init_fom_eval()
            self.stop=False
            # Start fitting in a new thread
            _thread.start_new_thread(self.optimize, ())
            # For debugging
            # self.optimize()
            self.text_output('Starting the fit...')
            # self.running = True
            return True
        else:
            self.text_output('Fit is already running, stop and then start')
            return False

    def stop_fit(self):
        '''
        Stops the fit if it has been started in a seperate theres 
        by start_fit.
        '''
        # If not running stop
        if self.running:
            self.stop=True
            self.text_output('Trying to stop the fit...')
        else:
            self.text_output('The fit is not running')

    def resume_fit(self, model):
        '''
        Resumes the fitting if has been stopped with stop_fit.
        '''
        if not self.running:
            self.stop=False
            self.connect_model(model)
            self.init_fom_eval()
            n_dim_old=self.n_dim
            if self.n_dim==n_dim_old:
                _thread.start_new_thread(self.optimize, ())
                self.text_output('Restarting the fit...')
                self.running=True
                return True
            else:
                self.text_output('The number of parameters has changed'
                                 ' restart the fit.')
                return False
        else:
            self.text_output('Fit is already running, stop and then start')
            return False

    def optimize(self):
        """Method that does the optimization.

        Note that this method does not run in a separate thread.
        For threading use start_fit, stop_fit and resume_fit instead.
        """
        if self.use_mpi:
            self.optimize_mpi()
        else:
            self.optimize_standard()

    def optimize_standard(self):
        '''
        Method implementing the main loop of the differential evolution
        algorithm. Note that this method does not run in a separate thread.
        For threading use start_fit, stop_fit and resume_fit instead.
        '''

        self.text_output('Calculating start FOM ...')
        self.running=True
        self.error=False
        self.n_fom=0
        ## Old leftovers before going parallel
        # self.fom_vec = [self.calc_fom(vec) for vec in self.pop_vec]
        # [self.par_evals.append(vec, axis = 0) for vec in self.pop_vec]
        # [self.fom_evals.append(vec) for vec in self.fom_vec]
        # New parallel calcualtions
        self.trial_vec=self.pop_vec[:]
        self.eval_fom()
        [self.par_evals.append(vec, axis=0) for vec in self.pop_vec]
        [self.fom_evals.append(vec) for vec in self.trial_fom]
        self.fom_vec=self.trial_fom[:]
        # print self.fom_vec
        best_index=argmin(self.fom_vec)
        # print self.fom_vec
        # print best_index
        self.best_vec=copy(self.pop_vec[best_index])
        # print self.best_vec
        self.best_fom=self.fom_vec[best_index]
        # print self.best_fom
        if len(self.fom_log)==0:
            self.fom_log=r_[self.fom_log, \
                            [[len(self.fom_log), self.best_fom]]]
        # Flag to keep track if there has been any improvemnts
        # in the fit - used for updates
        self.new_best=True

        self.text_output('Going into optimization ...')

        # Update the plot data for any gui or other output
        self.plot_output(self)
        self.parameter_output(self)

        # Just making gen live in this scope as well...
        gen=self.fom_log[-1, 0]
        for gen in range(int(self.fom_log[-1, 0])+1, self.max_gen+int(self.fom_log[-1, 0])+1):
            if self.stop:
                break

            t_start=time.time()

            self.init_new_generation(gen)

            # Create the vectors who will be compared to the 
            # population vectors
            [self.create_trial(index) for index in range(self.n_pop)]
            self.eval_fom()
            # Calculate the fom of the trial vectors and update the population
            [self.update_pop(index) for index in range(self.n_pop)]

            # Add the evaluation to the logging
            [self.par_evals.append(vec, axis=0) for vec in self.trial_vec]
            [self.fom_evals.append(vec) for vec in self.trial_fom]

            # Add the best value to the fom log
            self.fom_log=r_[self.fom_log, [[len(self.fom_log), self.best_fom]]]

            # Let the model calculate the simulation of the best.
            sim_fom=self.calc_sim(self.best_vec)

            # Sanity of the model does the simualtions fom agree with
            # the best fom
            if abs(sim_fom-self.best_fom)>self.fom_allowed_dis:
                self.text_output('Disagrement between two different fom'
                                 ' evaluations')
                self.error=('The disagreement between two subsequent '
                            'evaluations is larger than %s. Check the '
                            'model for circular assignments.'
                            %self.fom_allowed_dis)
                break

            # Update the plot data for any gui or other output
            self.plot_output(self)
            self.parameter_output(self)

            # limit the length of each iteration in parallel processing
            # at least on windows there is no issue with fast iterations in single thread
            to_sleep=self.sleep_time-(time.time()-t_start)
            if to_sleep>0:
                time.sleep(to_sleep)

            # Time measurent to track the speed
            t=time.time()-t_start
            if t>0:
                speed=self.n_pop/t
            else:
                speed=999999
            self.text_output('FOM: %.3f Generation: %d Speed: %.1f'% \
                             (self.best_fom, gen, speed))

            self.new_best=False
            # Do an autosave if activated and the interval is coorect
            if gen%self.autosave_interval==0 and self.use_autosave:
                self.autosave()

        if not self.error:
            self.text_output('Stopped at Generation: %d after %d fom evaluations...'%(gen, gen*self.n_pop))

        # Lets clean up and delete our pool of workers
        if self.use_parallel_processing:
            self.dismount_parallel()
        self.eval_fom=None

        # Now the optimization has stopped
        self.running=False

        # Run application specific clean-up actions
        self.fitting_ended(self)

    def optimize_mpi(self):
        '''
        Method implementing the main loop of the differential evolution
        algorithm using mpi. This should only be used from the command line.
        The gui can not handle to use mpi.
        '''

        if rank==0:
            self.text_output('Calculating start FOM ...')
        self.running=True
        self.error=False
        self.n_fom=0
        # Old leftovers before going parallel
        self.fom_vec=[self.calc_fom(vec) for vec in self.pop_vec]
        [self.par_evals.append(vec, axis=0) \
         for vec in self.pop_vec]
        [self.fom_evals.append(vec) for vec in self.fom_vec]
        # print self.fom_vec
        best_index=argmin(self.fom_vec)
        # print self.fom_vec
        # print best_index
        self.best_vec=copy(self.pop_vec[best_index])
        # print self.best_vec
        self.best_fom=self.fom_vec[best_index]
        # print self.best_fom
        if len(self.fom_log)==0:
            self.fom_log=r_[self.fom_log, \
                            [[len(self.fom_log), self.best_fom]]]
        # Flag to keep track if there has been any improvemnts
        # in the fit - used for updates
        self.new_best=True

        if rank==0:
            self.text_output('Going into optimization ...')

        # Update the plot data for any gui or other output
        self.plot_output(self)
        self.parameter_output(self)

        # Just making gen live in this scope as well...
        gen=self.fom_log[-1, 0]
        for gen in range(int(self.fom_log[-1, 0])+1, self.max_gen \
                                                     +int(self.fom_log[-1, 0])+1):
            if self.stop:
                break
            t_start=time.time()

            self.init_new_generation(gen)

            # Create the vectors who will be compared to the
            # population vectors
            if rank==0:
                [self.create_trial(index) for index in range(self.n_pop)]
                tmp_trial_vec=self.trial_vec
            else:
                tmp_trial_vec=0
            tmp_trial_vec=comm.bcast(tmp_trial_vec, root=0)
            self.trial_vec=tmp_trial_vec
            self.eval_fom()
            tmp_fom=self.trial_fom
            comm.Barrier()

            # collect foms and reshape them and set the completed tmp_fom to trial_fom
            tmp_fom=comm.gather(tmp_fom, root=0)
            if rank==0:
                tmp_fom_list=[]
                for i in list(tmp_fom):
                    tmp_fom_list=tmp_fom_list+i
                tmp_fom=tmp_fom_list

            tmp_fom=comm.bcast(tmp_fom, root=0)
            self.trial_fom=array(tmp_fom).reshape(self.n_pop, )

            [self.update_pop(index) for index in range(self.n_pop)]

            # Calculate the fom of the trial vectors and update the population
            if rank==0:
                # Add the evaluation to the logging
                [self.par_evals.append(vec, axis=0) for vec in self.trial_vec]
                [self.fom_evals.append(vec) for vec in self.trial_fom]

                # Add the best value to the fom log
                self.fom_log=r_[self.fom_log, [[len(self.fom_log), self.best_fom]]]

                # Let the model calculate the simulation of the best.
                sim_fom=self.calc_sim(self.best_vec)

                # Sanity of the model does the simualtions fom agree with
                # the best fom
                if abs(sim_fom-self.best_fom)>self.fom_allowed_dis and rank==0:
                    self.text_output('Disagrement between two different fom'
                                     ' evaluations')
                    self.error=('The disagreement between two subsequent '
                                'evaluations is larger than %s. Check the '
                                'model for circular assignments.'
                                %self.fom_allowed_dis)
                    break

                # Update the plot data for any gui or other output
                self.plot_output(self)
                self.parameter_output(self)

                # Let the optimization sleep for a while
                # time.sleep(self.sleep_time)

                # Time measurent to track the speed
                t=time.time()-t_start
                if t>0:
                    speed=self.n_pop/t
                else:
                    speed=999999
                if rank==0:
                    self.text_output('FOM: %.3f Generation: %d Speed: %.1f'%
                                     (self.best_fom, gen, speed))

                self.new_best=False
                # Do an autosave if activated and the interval is coorect
                if gen%self.autosave_interval==0 and self.use_autosave:
                    self.autosave()

        if rank==0:
            if not self.error:
                self.text_output('Stopped at Generation: %d after %d fom evaluations...'%(gen, gen*self.n_pop))

        # Lets clean up and delete our pool of workers

        self.eval_fom=None

        # Now the optimization has stopped
        self.running=False

        # Run application specific clean-up actions
        self.fitting_ended(self)

    def calc_fom(self, vec):
        '''
        Function to calcuate the figure of merit for parameter vector 
        vec.
        '''
        model=self.model
        model.limit_fit_range, model.fit_xmin, model.fit_xmax=(
            self.limit_fit_range,
            self.fit_xmin,
            self.fit_xmax)

        # Set the parameter values
        list(map(lambda func, value: func(value), self.par_funcs, vec))
        fom=self.model.evaluate_fit_func()
        self.n_fom+=1
        return fom

    def calc_trial_fom(self):
        '''
        Function to calculate the fom values for the trial vectors
        '''
        model=self.model
        model.limit_fit_range, model.fit_xmin, model.fit_xmax=(
            self.limit_fit_range,
            self.fit_xmin,
            self.fit_xmax)

        self.trial_fom=[self.calc_fom(vec) for vec in self.trial_vec]

    def calc_sim(self, vec):
        ''' calc_sim(self, vec) --> None
        Function that will evaluate the the data points for
        parameters in vec.
        '''
        model=self.model
        model.limit_fit_range, model.fit_xmin, model.fit_xmax=(
            self.limit_fit_range,
            self.fit_xmin,
            self.fit_xmax)
        # Set the paraemter values
        list(map(lambda func, value: func(value), self.par_funcs, vec))

        self.model.evaluate_sim_func()
        return self.model.fom

    def setup_parallel(self):
        '''setup_parallel(self) --> None
        
        setup for parallel proccesing. Creates a pool of workers with
        as many cpus there is available
        '''
        # check if CUDA has been activated
        from genx.models.lib import paratt
        use_cuda=paratt.Refl.__module__.rsplit('.',1)[1]=='paratt_cuda'
        self.pool=processing.Pool(processes=self.processes,
                                  initializer=parallel_init,
                                  initargs=(self.model.pickable_copy(), use_cuda))
        self.text_output("Starting a pool with %i workers ..."%(self.processes,))
        time.sleep(1.0)
        # print "Starting a pool with ", self.processes, " workers ..."

    def setup_parallel_mpi(self):
        """Inits the number or process used for mpi.
        """

        if rank==0:
            self.text_output("Inits mpi with %i processes ..."%(size,))
        parallel_init(self.model.pickable_copy())
        time.sleep(0.1)

    def dismount_parallel(self):
        ''' dismount_parallel(self) --> None
        Used to close the pool and all its processes
        '''
        self.pool.close()
        self.pool.join()

        # del self.pool

    def calc_trial_fom_parallel(self):
        '''calc_trial_fom_parallel(self) --> None
        
        Function to calculate the fom in parallel using the pool
        '''
        model=self.model
        model.limit_fit_range, model.fit_xmin, model.fit_xmax=(
            self.limit_fit_range,
            self.fit_xmin,
            self.fit_xmax)

        self.trial_fom=self.pool.map(parallel_calc_fom, self.trial_vec, chunksize=self.chunksize)

    def calc_trial_fom_parallel_mpi(self):
        """ Function to calculate the fom in parallel using mpi
        """
        model=self.model
        model.limit_fit_range, model.fit_xmin, model.fit_xmax=(
            self.limit_fit_range,
            self.fit_xmin,
            self.fit_xmax)

        step_len=int(len(self.trial_vec)/size)
        remainder=int(len(self.trial_vec)%size)
        left, right=0, 0

        if rank<=remainder-1:
            left=rank*(step_len+1)
            right=(rank+1)*(step_len+1)-1
        elif rank>remainder-1:
            left=remainder*(step_len+1)+(rank-remainder)*step_len
            right=remainder*(step_len+1)+(rank-remainder+1)*step_len-1
        fom_temp=[]

        for i in range(left, right+1):
            fom_temp.append(parallel_calc_fom(self.trial_vec[i]))

        self.trial_fom=fom_temp

    # noinspection PyArgumentList
    def calc_error_bar(self, index, fom_level):
        '''calc_error_bar(self, parameter) --> (error_bar_low, error_bar_high)
        
        Calculates the errorbar for one parameter number index. 
        returns a float tuple with the error bars. fom_level is the 
        level which is the upperboundary of the fom is allowed for the
        calculated error.
        '''
        # print self.par_evals.shape, self.par_evals
        # print self.fom_evals.shape, self.fom_evals
        if self.setup_ok:  # and len(self.par_evals) != 0:
            par_values=self.par_evals[:, index]
            # print (self.fom_evals < fom_level).sum()
            values_under_level=compress(self.fom_evals[:]<fom_level*self.best_fom, par_values)
            # print values_under_level
            error_bar_low=values_under_level.min()-self.best_vec[index]
            error_bar_high=values_under_level.max()-self.best_vec[index]
            return error_bar_low, error_bar_high
        else:
            raise ErrorBarError()

    def init_new_generation(self, gen):
        ''' Function that is called every time a new generation starts'''
        pass

    def standard_init_new_generation(self, gen):
        ''' Function that is called every time a new generation starts'''
        pass

    def standard_update_pop(self, index):
        '''
        Function to update population vector index. calcs the figure of merit
        and compares it to the current population vector and also checks
        if it is better than the current best.
        '''
        # fom = self.calc_fom(self.trial_vec[index])
        fom=self.trial_fom[index]
        if fom<self.fom_vec[index]:
            self.pop_vec[index]=self.trial_vec[index].copy()
            self.fom_vec[index]=fom
            if fom<self.best_fom:
                self.new_best=True
                self.best_vec=self.trial_vec[index].copy()
                self.best_fom=fom

    # noinspection PyArgumentList
    def simplex_old_init_new_generation(self, gen):
        '''It will run the simplex method every simplex_interval
             generation with a fracitonal step given by simple_step 
             on the best indivual as well a random fraction of simplex_n individuals.
        '''
        iprint('Inits new generation')
        if gen%self.simplex_interval==0:
            spread=array(self.trial_vec).max(0)-array(self.trial_vec).min(0)
            simp=Simplex(self.calc_fom, self.best_vec, spread*self.simplex_step)
            iprint('Starting simplex run for best vec')
            new_vec, err, iter=simp.minimize(epsilon=self.best_fom/self.simplex_rel_epsilon,
                                             maxiters=self.simplex_max_iter)
            iprint('FOM improvement: ', self.best_fom-err)

            if self.use_boundaries:
                # Check so that the parameters lie indside the bounds
                ok=bitwise_and(self.par_max>new_vec, self.par_min<new_vec)
                # If not inside make a random re-initilazation of that parameter
                new_vec=where(ok, new_vec, random.rand(self.n_dim)* \
                              (self.par_max-self.par_min)+self.par_min)

            new_fom=self.calc_fom(new_vec)
            if new_fom<self.best_fom:
                self.best_fom=new_fom
                self.best_vec=new_vec
                self.pop_vec[0]=new_vec
                self.fom_vec[0]=self.best_fom
                self.new_best=True

            # Apply the simplex to a simplex_n memebers (0-1)
            for index1 in random_mod.sample(range(len(self.pop_vec)),
                                            int(len(self.pop_vec)*self.simplex_n)):
                iprint('Starting simplex run for member: ', index1)
                mem=self.pop_vec[index1]
                mem_fom=self.fom_vec[index1]
                simp=Simplex(self.calc_fom, mem, spread*self.simplex_step)
                new_vec, err, iter=simp.minimize(epsilon=self.best_fom/self.simplex_rel_epsilon,
                                                 maxiters=self.simplex_max_iter)
                if self.use_boundaries:
                    # Check so that the parameters lie indside the bounds
                    ok=bitwise_and(self.par_max>new_vec, self.par_min<new_vec)
                    # If not inside make a random re-initilazation of that parameter
                    new_vec=where(ok, new_vec, random.rand(self.n_dim)* \
                                  (self.par_max-self.par_min)+self.par_min)

                new_fom=self.calc_fom(new_vec)
                if new_fom<mem_fom:
                    self.pop_vec[index1]=new_vec
                    self.fom_vec[index1]=new_fom
                    if new_fom<self.best_fom:
                        self.best_fom=new_fom
                        self.best_vec=new_vec
                        self.new_best=True

    # noinspection PyArgumentList
    def simplex_init_new_generation(self, gen):
        '''It will run the simplex method every simplex_interval
             generation with a fracitonal step given by simple_step 
             on the simplex_n*n_pop best individuals.
        '''
        iprint('Inits new generation')
        if gen%self.simplex_interval==0:
            spread=array(self.trial_vec).max(0)-array(self.trial_vec).min(0)

            indices=argsort(self.fom_vec)
            n_ind=int(self.n_pop*self.simplex_n)
            if n_ind==0:
                n_ind=1
            # Apply the simplex to a simplex_n memebers (0-1)
            for index1 in indices[:n_ind]:
                self.text_output('Starting simplex run for member: %d'%index1)
                mem=self.pop_vec[index1].copy()
                mem_fom=self.fom_vec[index1]
                simp=Simplex(self.calc_fom, mem, spread*self.simplex_step)
                new_vec, err, iter=simp.minimize(epsilon=self.best_fom/self.simplex_rel_epsilon,
                                                 maxiters=self.simplex_max_iter)
                if self.use_boundaries:
                    # Check so that the parameters lie indside the bounds
                    ok=bitwise_and(self.par_max>new_vec, self.par_min<new_vec)
                    # If not inside make a random re-initilazation of that parameter
                    new_vec=where(ok, new_vec, random.rand(self.n_dim)* \
                                  (self.par_max-self.par_min)+self.par_min)

                new_fom=self.calc_fom(new_vec)
                if new_fom<mem_fom:
                    self.pop_vec[index1]=new_vec.copy()
                    self.fom_vec[index1]=new_fom
                    if new_fom<self.best_fom:
                        self.best_fom=new_fom
                        self.best_vec=new_vec.copy()
                        self.new_best=True

    def simplex_best_1_bin(self, index):
        return self.best_1_bin(index)

    def jade_update_pop(self, index):
        ''' A modified update pop to handle the JADE variation of Differential evoluion'''
        fom=self.trial_fom[index]
        if fom<self.fom_vec[index]:
            self.pop_vec[index]=self.trial_vec[index].copy()
            self.fom_vec[index]=fom
            self.updated_kr.append(self.kr_vec[index])
            self.updated_km.append(self.km_vec[index])
            if fom<self.best_fom:
                self.new_best=True
                self.best_vec=self.trial_vec[index].copy()
                self.best_fom=fom

    def jade_init_new_generation(self, gen):
        ''' A modified generation update for jade'''
        # print 'inits generation: ', gen, self.n_pop
        if gen>1:
            updated_kms=array(self.updated_km)
            updated_krs=array(self.updated_kr)
            if len(updated_kms)!=0:
                self.km=(1.0-self.c)*self.km+self.c*sum(updated_kms**2)/sum(updated_kms)
                self.kr=(1.0-self.c)*self.kr+self.c*mean(updated_krs)
        self.km_vec=abs(self.km+random.standard_cauchy(self.n_pop)*0.1)
        self.kr_vec=self.kr+random.normal(size=self.n_pop)*0.1
        # print self.km_vec, self.kr_vec
        iprint('km: ', self.km, ', kr: ', self.kr)
        # self.km_vec = (self.km_vec >= 1)*1 + (self.km_vec < 1)*self.km_vec
        self.km_vec=where(self.km_vec>0, self.km_vec, 0)
        self.km_vec=where(self.km_vec<1, self.km_vec, 1)
        self.kr_vec=where(self.kr_vec>0, self.kr_vec, 0)
        self.kr_vec=where(self.kr_vec<1, self.kr_vec, 1)

        self.updated_kr=[]
        self.updated_km=[]

    def jade_best(self, index):
        vec=self.pop_vec[index]
        # Create mutation vector
        # Select two random vectors for the mutation
        index1=int(random.rand(1)*self.n_pop)
        index2=int(random.rand(1)*len(self.par_evals))
        # Make sure it is not the same vector 
        # while index2 == index1:
        #    index2 = int(random.rand(1)*self.n_pop)

        # Calculate the mutation vector according to the best/1 scheme
        # print len(self.km_vec), index, len(self.par_evals),  index2
        mut_vec=vec+self.km_vec[index]*(self.best_vec-vec)+self.km_vec[index]*(
                self.pop_vec[index1]-self.par_evals[index2])

        # Binomial test to detemine which parameters to change
        # given by the recombination constant kr
        recombine=random.rand(self.n_dim)<self.kr_vec[index]
        # Make sure at least one parameter is changed
        recombine[int(random.rand(1)*self.n_dim)]=1
        # Make the recombination
        trial=where(recombine, mut_vec, vec)

        # Implementation of constrained optimization
        if self.use_boundaries:
            # Check so that the parameters lie indside the bounds
            ok=bitwise_and(self.par_max>trial, self.par_min<trial)
            # If not inside make a random re-initilazation of that parameter
            trial=where(ok, trial, random.rand(self.n_dim)* \
                        (self.par_max-self.par_min)+self.par_min)
        self.trial_vec[index]=trial
        # return trial

    def best_1_bin(self, index):
        '''best_1_bin(self, vec) --> trial [1D array]
        
        The default create_trial function for this class. 
        uses the best1bin method to create a new vector from the population.
        '''
        vec=self.pop_vec[index]
        # Create mutation vector
        # Select two random vectors for the mutation
        index1=int(random.rand(1)*self.n_pop)
        index2=int(random.rand(1)*self.n_pop)
        # Make sure it is not the same vector 
        while index2==index1:
            index2=int(random.rand(1)*self.n_pop)

        # Calculate the mutation vector according to the best/1 scheme
        mut_vec=self.best_vec+self.km*(
                self.pop_vec[index1]-self.pop_vec[index2])

        # Binomial test to detemine which parameters to change
        # given by the recombination constant kr
        recombine=random.rand(self.n_dim)<self.kr
        # Make sure at least one parameter is changed
        recombine[int(random.rand(1)*self.n_dim)]=1
        # Make the recombination
        trial=where(recombine, mut_vec, vec)

        # Implementation of constrained optimization
        if self.use_boundaries:
            # Check so that the parameters lie indside the bounds
            ok=bitwise_and(self.par_max>trial, self.par_min<trial)
            # If not inside make a random re-initilazation of that parameter
            trial=where(ok, trial, random.rand(self.n_dim)* \
                        (self.par_max-self.par_min)+self.par_min)

        self.trial_vec[index]=trial
        # return trial

    def best_either_or(self, index):
        '''best_either_or(self, vec) --> trial [1D array]
        
        The either/or scheme for creating a trial. Using the best vector
        as base vector.
        '''
        vec=self.pop_vec[index]
        # Create mutation vector
        # Select two random vectors for the mutation
        index1=int(random.rand(1)*self.n_pop)
        index2=int(random.rand(1)*self.n_pop)
        # Make sure it is not the same vector 
        while index2==index1:
            index2=int(random.rand(1)*self.n_pop)

        if random.rand(1)<self.pf:
            # Calculate the mutation vector according to the best/1 scheme
            trial=self.best_vec+self.km*(
                    self.pop_vec[index1]-self.pop_vec[index2])
        else:
            # Trying something else out more like normal recombination
            trial=vec+self.kr*(
                    self.pop_vec[index1]+self.pop_vec[index2]-2*vec)

        # Implementation of constrained optimization
        if self.use_boundaries:
            # Check so that the parameters lie indside the bounds
            ok=bitwise_and(self.par_max>trial, self.par_min<trial)
            # If not inside make a random re-initilazation of that parameter
            trial=where(ok, trial, random.rand(self.n_dim)* \
                        (self.par_max-self.par_min)+self.par_min)
        self.trial_vec[index]=trial
        # return trial

    def rand_1_bin(self, index):
        '''best_1_bin(self, vec) --> trial [1D array]
        
        The default create_trial function for this class. 
        uses the best1bin method to create a new vector from the population.
        '''
        vec=self.pop_vec[index]
        # Create mutation vector
        # Select three random vectors for the mutation
        index1=int(random.rand(1)*self.n_pop)
        index2=int(random.rand(1)*self.n_pop)
        # Make sure it is not the same vector 
        while index2==index1:
            index2=int(random.rand(1)*self.n_pop)
        index3=int(random.rand(1)*self.n_pop)
        while index3==index1 or index3==index2:
            index3=int(random.rand(1)*self.n_pop)

        # Calculate the mutation vector according to the rand/1 scheme
        mut_vec=self.pop_vec[index3]+self.km*(
                self.pop_vec[index1]-self.pop_vec[index2])

        # Binomial test to detemine which parameters to change
        # given by the recombination constant kr
        recombine=random.rand(self.n_dim)<self.kr
        # Make sure at least one parameter is changed
        recombine[int(random.rand(1)*self.n_dim)]=1
        # Make the recombination
        trial=where(recombine, mut_vec, vec)

        # Implementation of constrained optimization
        if self.use_boundaries:
            # Check so that the parameters lie indside the bounds
            ok=bitwise_and(self.par_max>trial, self.par_min<trial)
            # If not inside make a random re-initilazation of that parameter
            trial=where(ok, trial, random.rand(self.n_dim)* \
                        (self.par_max-self.par_min)+self.par_min)
        self.trial_vec[index]=trial
        # return trial

    def rand_either_or(self, index):
        '''rand_either_or(self, vec) --> trial [1D array]
        
        random base vector either/or trial scheme
        '''
        vec=self.pop_vec[index]
        # Create mutation vector
        # Select two random vectors for the mutation
        index1=int(random.rand(1)*self.n_pop)
        index2=int(random.rand(1)*self.n_pop)
        # Make sure it is not the same vector 
        while index2==index1:
            index2=int(random.rand(1)*self.n_pop)
        index0=int(random.rand(1)*self.n_pop)
        while index0==index1 or index0==index2:
            index0=int(random.rand(1)*self.n_pop)

        if random.rand(1)<self.pf:
            # Calculate the mutation vector according to the best/1 scheme
            trial=self.pop_vec[index0]+self.km*(
                    self.pop_vec[index1]-self.pop_vec[index2])
        else:
            # Calculate a continous recomibination
            # Trying something else out more like normal recombination
            trial=self.pop_vec[index0]+self.kr*(
                    self.pop_vec[index1]+self.pop_vec[index2]-2*self.pop_vec[index0])
            # trial = vec + self.kr*(self.pop_vec[index1]\
            #        + self.pop_vec[index2] - 2*vec)

        # Implementation of constrained optimization
        if self.use_boundaries:
            # Check so that the parameters lie indside the bounds
            ok=bitwise_and(self.par_max>trial, self.par_min<trial)
            # If not inside make a random re-initilazation of that parameter
            trial=where(ok, trial, random.rand(self.n_dim)* \
                        (self.par_max-self.par_min)+self.par_min)
        self.trial_vec[index]=trial
        # return trial

    # Different function for acessing and setting parameters that
    # the user should have control over.

    def set_text_output_func(self, func):
        '''set_text_output_func(self, func) --> None
        
        Set the output function for the text output from the optimizer.
        Should be a function that takes a string as input argument.
        The default function is a simple print statement.
        '''
        self.text_output=func

    def set_plot_output_func(self, func):
        '''set_plot_output_func(self, func) --> None

        Set the output function for the plot output from the optimizer.
        Should take the an instance of solver as input.
        The default function is no output whatsoever
        '''
        self.plot_output=func

    def set_parameter_output_func(self, func):
        '''set_parameters_output_func(self, func) --> None

        Set the output function for the parameters output from the optimizer.
        Should take the an instance of solver as input.
        The default function is no output whatsoever
        '''
        self.parameter_output=func

    def set_fitting_ended_func(self, func):
        '''set_fitting_ended_func(self, func) --> None

        Set the function when the optimizer has finsihed the fitting.
        Should take the an instance of solver as input.
        The default function is no output whatsoever
        '''
        self.fitting_ended=func

    def set_autosave_func(self, func):
        '''set_autosave_func(self, func) --> None
        
        Set the function that the optimizer uses to do an autosave
        of the current fit. Function func should not take any arguments.
        '''
        self.autosave=func

    # Some get functions

    def get_model(self):
        '''get_model(self) --> model
        Getter that returns the model in use in solver.
        '''
        return self.model

    def get_fom_log(self):
        '''get_fom_log(self) -->  fom [array]
        Returns the fom as a fcn of iteration in an array. 
        Last element last fom value
        '''
        return array(self.fom_log)

    def get_create_trial(self, index=False):
        '''get_create_trial(self, index = False) --> string or int
        
        returns the current create trial function name if index is False as
        a string or as index in the mutation_schemes list.
        '''
        pos=self.mutation_schemes.index(self.create_trial)
        if index:
            # return the position
            return pos
        else:
            # return the name
            return self.mutation_schemes[pos].__name__

    def set_km(self, val):
        '''set_km(self, val) --> None
        '''
        self.km=val

    def set_kr(self, val):
        '''set_kr(self, val) --> None
        '''
        self.kr=val

    def set_create_trial(self, val):
        '''set_create_trial(self, val) --> None
        
        Raises LookupError if the value val [string] does not correspond
        to a mutation scheme/trial function
        '''
        # Get the names of the available functions
        names=[f.__name__ for f in self.mutation_schemes]
        # Find the postion of val

        pos=names.index(val)
        self.create_trial=self.mutation_schemes[pos]
        if val=='jade_best':
            self.update_pop=self.jade_update_pop
            self.init_new_generation=self.jade_init_new_generation
        elif val=='simplex_best_1_bin':
            self.init_new_generation=self.simplex_init_new_generation
            self.update_pop=self.standard_update_pop
        else:
            self.init_new_generation=self.standard_init_new_generation
            self.update_pop=self.standard_update_pop

    def set_pop_mult(self, val):
        '''set_pop_mult(self, val) --> None
        '''
        self.pop_mult=val

    def set_pop_size(self, val):
        '''set_pop_size(self, val) --> None
        '''
        self.pop_size=int(val)

    def set_max_generations(self, val):
        '''set_max_generations(self, val) --> None
        '''
        self.max_generations=int(val)

    def set_max_generation_mult(self, val):
        '''set_max_generation_mult(self, val) --> None
        '''
        self.max_generation_mult=val

    def set_sleep_time(self, val):
        '''set_sleep_time(self, val) --> None
        '''
        self.sleep_time=val

    def set_max_log(self, val):
        '''Sets the maximum number of logged elements
        '''
        self.max_log=val

    def set_use_pop_mult(self, val):
        '''set_use_pop_mult(self, val) --> None
        '''
        self.use_pop_mult=val

    def set_use_max_generations(self, val):
        '''set_use_max_generations(self, val) --> None
        '''
        self.use_max_generations=val

    def set_use_start_guess(self, val):
        '''set_use_start_guess(self, val) --> None
        '''
        self.use_start_guess=val

    def set_use_boundaries(self, val):
        '''set_use_boundaries(self, val) --> None
        '''
        self.use_boundaries=val

    def set_use_autosave(self, val):
        '''set_use_autosave(self, val) --> None
        '''
        self.use_autosave=val

    def set_autosave_interval(self, val):
        '''set_autosave_interval(self, val) --> None
        '''
        self.autosave_interval=int(val)

    def set_use_parallel_processing(self, val):
        '''set_use_parallel_processing(self, val) --> None
        '''
        if __parallel_loaded__:
            self.use_parallel_processing=val
            self.use_mpi=False if val else self.use_mpi
        else:
            self.use_parallel_processing=False

    def set_use_mpi(self, val):
        """Sets if mpi should use for parallel optimization"""
        if __mpi_loaded__:
            self.use_mpi=val
            self.use_parallel_processing=False if val else self.use_parallel_processing
        else:
            self.use_mpi=False

    def set_processes(self, val):
        '''set_processes(self, val) --> None
        '''
        self.processes=int(val)

    def set_chunksize(self, val):
        '''set_chunksize(self, val) --> None
        '''
        self.chunksize=int(val)

    def set_fom_allowed_dis(self, val):
        '''set_chunksize(self, val) --> None
        '''
        self.fom_allowed_dis=float(val)

    def __repr__(self):
        output="Differential Evolution Optimizer:\n"
        for gname, group in self.parameter_groups:
            output+='    %s:\n'%gname
            for attr in group:
                output+='        %-30s %s\n'%(attr, getattr(self, attr))
        return output

    @property
    def widget(self):
        return self._repr_ipyw_()

    def _repr_ipyw_(self):
        import ipywidgets as ipw
        entries=[]
        for gname, group in self.parameter_groups:
            gentries=[ipw.HTML("<b>%s:</b>"%gname)]
            for attr in group:
                val=eval('self.%s'%attr, globals(), locals())
                if type(val) is bool:
                    item=ipw.Checkbox(value=val, indent=False, description=attr, layout=ipw.Layout(width='24ex'))
                    entry=item
                elif type(val) is int:
                    entry=ipw.IntText(value=val, layout=ipw.Layout(width='18ex'))
                    item=ipw.VBox([ipw.Label(attr), entry])
                elif type(val) is float:
                    entry=ipw.FloatText(value=val, layout=ipw.Layout(width='18ex'))
                    item=ipw.VBox([ipw.Label(attr), entry])
                elif attr=='method':
                    entry=ipw.Dropdown(value=val, options=self.methods, layout=ipw.Layout(width='18ex'))
                    item=ipw.VBox([ipw.Label(attr), entry])
                else:
                    entry=ipw.Text(value=val, layout=ipw.Layout(width='14ex'))
                    item=ipw.VBox([ipw.Label(attr), entry])
                entry.change_item=attr
                entry.observe(self._ipyw_change, names='value')
                gentries.append(item)
            entries.append(ipw.VBox(gentries, layout=ipw.Layout(width='26ex')))
        return ipw.VBox([ipw.HTML("<h3>Optimizer Settings:</h3>"), ipw.HBox(entries)])

    def _ipyw_change(self, change):
        exec('self.%s=change.new'%change.owner.change_item)

# ==============================================================================
# Functions that is needed for parallel processing!
def parallel_init(model_copy, use_cuda):
    '''parallel_init(model_copy) --> None
    
    parallel initilization of a pool of processes. The function takes a
    pickle safe copy of the model and resets the script module and the compiles
    the script and creates function to set the variables.
    '''
    if use_cuda:
        # activate cuda in subprocesses
        from genx.models.lib import paratt_cuda
        from genx.models.lib import neutron_cuda
        from models.lib import paratt, neutron_refl
        paratt.Refl=paratt_cuda.Refl
        paratt.ReflQ=paratt_cuda.ReflQ
        paratt.Refl_nvary2=paratt_cuda.Refl_nvary2
        neutron_refl.Refl=neutron_cuda.Refl
        from genx.models.lib import paratt, neutron_refl
        paratt.Refl=paratt_cuda.Refl
        paratt.ReflQ=paratt_cuda.ReflQ
        paratt.Refl_nvary2=paratt_cuda.Refl_nvary2
        neutron_refl.Refl=neutron_cuda.Refl
    global model, par_funcs
    model=model_copy
    model._reset_module()
    model.simulate()
    (par_funcs, start_guess, par_min, par_max)=model.get_fit_pars()
    # print 'Sucess!'

def parallel_calc_fom(vec):
    '''parallel_calc_fom(vec) --> fom (float)
    
    function that is used to calculate the fom in a parallel process.
    It is a copy of calc_fom in the DiffEv class
    '''
    global model, par_funcs
    # print 'Trying to set parameters'
    # set the parameter values in the model
    list(map(lambda func, value: func(value), par_funcs, vec))
    # print 'Trying to evaluate'
    # evaluate the model and calculate the fom
    fom=model.evaluate_fit_func()

    return fom

# ==============================================================================
def default_text_output(text):
    iprint(text)
    sys.stdout.flush()

def default_plot_output(solver):
    pass

def default_parameter_output(solver):
    pass

def default_fitting_ended(solver):
    pass

def defualt_autosave():
    pass

def _calc_fom(model, vec, par_funcs):
    '''
    Function to calcuate the figure of merit for parameter vector
    vec.
    '''
    # Set the paraemter values
    list(map(lambda func, value: func(value), par_funcs, vec))

    return model.evaluate_fit_func()

# ==============================================================================
# BEGIN: class CircBuffer
class CircBuffer:
    '''A buffer with a fixed length to store the logging data from the diffev 
    class. Initilized to a maximumlength after which it starts to overwrite
    the data again.
    '''

    def __init__(self, maxlen, buffer=None):
        '''Inits the class with a certain maximum length maxlen.
        '''
        self.maxlen=int(maxlen)
        self.pos=-1
        self.filled=False
        if buffer is None:
            self.buffer=zeros((self.maxlen,))
        else:
            if len(buffer)!=0:
                self.buffer=array(buffer).repeat(
                    ceil(self.maxlen/(len(buffer)*1.0)), 0)[:self.maxlen]
                self.pos=len(buffer)-1
            else:
                self.buffer=zeros((self.maxlen,)+buffer.shape[1:])

    def reset(self, buffer=None):
        '''Resets the buffer to the initial state
        '''
        self.pos=-1
        self.filled=False
        # self.buffer = buffer
        if buffer is None:
            self.buffer=zeros((self.maxlen,))
        else:
            if len(buffer)!=0:
                self.buffer=array(buffer).repeat(
                    ceil(self.maxlen/(len(buffer)*1.0)), 0)[:self.maxlen]
                self.pos=len(buffer)-1
            else:
                self.buffer=zeros((self.maxlen,)+buffer.shape[1:])

    def append(self, item, axis=None):
        '''Appends an element to the last position of the buffer
        '''
        new_pos=(self.pos+1)%self.maxlen
        if len(self.buffer)>=self.maxlen:
            if self.pos>=(self.maxlen-1):
                self.filled=True
            self.buffer[new_pos]=array(item).real
        else:
            self.buffer=append(self.buffer, item, axis=axis)
        self.pos=new_pos

    def array(self):
        '''returns an ordered array instead of the circular
        working version
        '''
        if self.filled:
            return r_[self.buffer[self.pos+1:], self.buffer[:self.pos+1]]
        else:
            return r_[self.buffer[:self.pos+1]]

    def copy_from(self, object):
        '''Add copy support
        '''
        if type(object)==type(array([])):
            self.buffer=object[-self.maxlen:]
            self.pos=len(self.buffer)-1
            self.filled=self.pos>=(self.maxlen-1)
        elif object.__class__==self.__class__:
            # Check if the buffer has been removed.
            if len(object.buffer)==0:
                self.__init__(object.maxlen, object.buffer)
            else:
                self.buffer=object.buffer.copy()
                self.maxlen=object.maxlen
                self.pos=object.pos
                try:
                    self.filled=object.filled
                except:
                    self.filled=False
        else:
            raise TypeError('CircBuffer support only copying from CircBuffer'
                            ' and arrays.')

    def __len__(self):
        if self.filled:
            return len(self.buffer)
        else:
            return (self.pos>0)*self.pos

    def __getitem__(self, key):
        return self.array().__getitem__(key)

# END: class CircBuffer
# ==============================================================================
class GenericError(Exception):
    ''' Just a empty class used for inheritance. Only useful
    to check if the errors are originating from the model library.
    All these errors are controllable. If they not originate from
    this class something has passed trough and that should be impossible '''
    pass

class ErrorBarError(GenericError):
    '''Error class for the fom evaluation'''

    def __init__(self):
        ''' __init__(self) --> None'''
        # self.error_message = error_message

    def __str__(self):
        text='Could not evaluate the error bars. A fit has to be made '+ \
             'before they can be calculated'
        return text

if __name__=='__main__':
    import h5py

    d=DiffEv()
    iprint(arange(10))
    d.fom_evals.copy_from(arange(10))
    iprint(d.fom_evals.buffer, d.fom_evals.pos, d.fom_evals.filled)
    d.km=10
    iprint(d.fom_evals.array())
    f=h5py.File('myfile.hdf5', 'w')
    dic=f.create_group('optimizer')
    d.write_h5group(dic)
    f.close()

    d=DiffEv()
    f=h5py.File('myfile.hdf5', 'r')
    dic=f['optimizer']
    d.read_h5group(dic)
    iprint(d.km)
    iprint(d.fom_evals.array(), d.fom_evals.pos, d.fom_evals.filled)
    f.close()
